/* WARNING: This file was automatically generated from ants/ANEPChannel.Java DO NOT EDIT *//* 
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */
/* 
 *  $Id: ANEPChannel.Java,v 1.6 1998/06/01 04:50:22 gback Exp $
 */

package ants ;
import nodeos .*;

import java.net.*;
import java.io.*;

/**
 * ANEPChannel: forwarding unrecognized ANEP packets
 * <p>
 * An ANEP Channel will provide default handling for ANEP packets
 * that are not of a recognized type.  The associated flow handle
 * is connected to the ANEPDispatcher's registerDefault method.
 *
 * @see InChannel
 *
 * @author Godmar Back
 * 	   David Wetherall
 */

public class ANEPChannel
  extends InChannel
{
  /**
   * Decode ANEP header and forward packet
   * <p>
   * Always returns null because no capsule is created.
   * 
   * @return null
   */
  protected Capsule decode(Xdr xdr) throws Exception {

    if (xdr.BYTE() != Channel.ANEPVERSION)
      throw new Exception("Wrong ANEP version");
    boolean discard = (xdr.BYTE() & 0x80) != 0;
    short typeID = xdr.SHORT();
    short headerLen = xdr.SHORT();
    xdr.index += Xdr.SHORT;     // Skip packet length field

    if (typeID != Channel.ANTSTYPE) {
      if (discard)
	throw new Exception("Unknown type discard");
      else {
	int dest = 0;
	while (xdr.index < headerLen * Xdr.INT) {
	  short flagsAndType = xdr.SHORT();
	  boolean optPrivate = (flagsAndType & 0x8000) != 0;
	  boolean optDiscard = (flagsAndType & 0x4000) != 0;	
	  short optType = (short)(xdr.SHORT() & 0x3FFF);
	  short optLen = xdr.SHORT();
	  if (!optPrivate) {
	    if (optType == Channel.DESTOPTTYPE) {
	      int scheme = xdr.INT();
	      if (scheme != Channel.IPV4SCHEME)
		throw new Exception("Unknown addressing scheme");
	      else {
		dest = xdr.INT();
	      }
	    } else if (optType == Channel.SRCOPTTYPE) {
	      xdr.index += (optLen - 1) * Xdr.INT;
	    } else if (optDiscard) 
	      throw new Exception("Unknown option discard");
	  } else {
	    xdr.index += (optLen - 1) * Xdr.INT;
	  }
	}

	RouteEntry r = client.getRoutes().get(dest);
	if (r != null) {
	  if (xdr.offset != 0) {
	    byte[] bytes = new byte[xdr.len];
	    System.arraycopy(xdr.buf, xdr.offset, bytes, 0, xdr.len);
	    client.link.send(bytes, xdr.len, r.addr);
	  } else
	    client.link.send(xdr.buf, xdr.len, r.addr);
	  throw new Exception("Unknown type forwarded");
	} else
	  throw new Exception("No route to destination while forwarding");
      }
    }
    // skip ANEP header
    xdr.index = headerLen * Xdr.INT;
    return null;
  }

  public ANEPChannel(Node node, String name) 
  {
    super(node, null, name, false);
  }
}

