/* 
 *  $Id: BuiltinProtocol.Java,v 1.2 1998/05/29 06:00:41 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */

package ANTS_PKG;

import java.util.*;
import UTILS_PKG.*;


/**
 * Base class for all builtin protocols.  Builtin protocols are 
 * not demand loaded and are known by "well-defined" names
 *
 * @author David Wetherall
 */

public class BuiltinProtocol extends Protocol {

  // we override this to mark the group as builtin
  protected void endGroupDefn() throws Exception {
    super.endGroupDefn();
    CodeGroup g = (CodeGroup)groups.lastElement();
    g.builtin = 1;
  }


  // we override this to set codeID from version only
  protected void addCapsule(String name, String version) throws Exception {
    CodeGroup g = (CodeGroup)groups.lastElement();
    Method m = new Method(name);    

    // change is here
    MD5 id = new MD5();
    id.update(version);
    m.codeID = TypeID.wrap(id.hash()).intern();

    m.group = g;
    g.meths.addElement(m);
    g.codeIDs.addElement(m.codeID);
    capsules.addElement(m);
  }

  // and this to catch a few errors
  protected void addCapsule(String name) throws Exception {
    throw new Exception("regular addCapsule for builtin protocol!");
  }

}

