/* WARNING: This file was automatically generated from ants/Capsule.Java DO NOT EDIT *//* 
 *  $Id: Capsule.Java,v 1.5 1998/05/29 20:51:45 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ants ;
import nodeos .*;

import utils .*;


/**
 * Base class for all types of capsules and their processing
 *
 * @author David Wetherall
 */


public abstract class Capsule {
  protected abstract byte[] mid();
  protected abstract byte[] pid();

  /** All capsules originate from some source node. */
  int src; // accessed directly by channel
  private int dst;

  /** Previous active node, required for code loading protocol */
  int previous; // accessed directly by channel

  public int getPrevious() {
    return previous;
  }

  int resources; // accessed directly by channel and nodecache

  public int getResources() {
    return resources;
  }

  Xdr xdr = null;
  boolean saveBuffer = false;

  final static protected int LVL = Entity.L[1]; 

  /** Access point to initialze capsule methodIDs */
  public static byte[] findMID(String name) {
    // redirect access point to the real implementation
    return Method.findMID(name);
  }

  /** Access point to initialze capsule groupIDs */
  public static byte[] findGID(String name) {
    // redirect access point to the real implementation
    return Method.findGID(name);
  }

  /** Access point to initialze capsule protocolIDs */
  public static byte[] findPID(String name) {
    // redirect access point to the real implementation
    return Method.findPID(name);
  }

  /** Access point to initialze capsule extension indexes */
  public static int findExtension(String name) {
    // redirect access point to the real implementation
    return Extension.findNamedExtension(name);
  }

  abstract public boolean evaluate(Node n);
  
  public int getSrc() {
    return src;
  }

  public int getDst() {
    return dst;
  }

  public void setDst(int dst) {
    this.dst = dst;
  }

  public int length() {
    return Xdr.TYPEID + Xdr.INT + Xdr.INT + Xdr.INT + Xdr.INT;
  }
  
  public Xdr encode() {
    Xdr xdr = new Xdr(this.xdr, this.xdr.index);
    xdr.index = 0;
    xdr.PUT(this.mid(), 0, Xdr.TYPEID);
    xdr.PUT(src);
    xdr.PUT(dst);
    xdr.PUT(previous);
    xdr.PUT(resources);
    return xdr;
  }

  public Xdr decode() {
    Xdr xdr = new Xdr(this.xdr, this.xdr.index);
    
    // skip over the MID field
    xdr.index += Xdr.TYPEID;

    src = xdr.INT();
    dst = xdr.INT();
    previous = xdr.INT();
    resources = xdr.INT();
    return xdr;
  }

  /** For bootstrapping capsule code loading */
  final static int peekPrevious(Buffer buf) {
    Xdr xdr = new Xdr(buf.getData(), (int)buf.getStart() 
	+ Xdr.TYPEID + Xdr.INT + Xdr.INT);
    return xdr.INT();
  }

  /** Prime a capsule created within the network for transmission */
  public final void prime(Capsule parent) {
    this.src = parent.src;
    int rnew = parent.resources--;
    if (rnew <= 0) throw new ResourceLimitException();
    this.resources = rnew;
  }

  /** Priveleged prime operation */
  final void prime(int src, int resources) {
    this.src = src;
    this.resources = resources;
  }

  public static Capsule create(Class cl, Xdr xdr) {
    Capsule c = null;
    try {
      c = (Capsule)(cl.newInstance());
    } catch (Exception e) {
      Entity.error("Class.newInstance failed for "+ cl);
    }
    c.xdr = xdr;
    c.decode();
    return c;
  }

  public Capsule() { }
}




