/* 
 *  $Id: Extension.Java,v 1.2 1998/05/29 06:00:49 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ANTS_PKG;

import java.util.*;
import UTILS_PKG.*;

/**
 * Shell of a node extension component
 *
 * @author David Wetherall
 */


public abstract class Extension
  extends ManagedObject
  implements Runnable
{
  final public static String[] defaults = { };

  private Node server;
  public Node thisNode() { return server; }

  private static int count = 0;
  private static Hashtable registry = new Hashtable();

  public static int findNamedExtension(String name) {
    Integer index = (Integer)registry.get(name);
    if (index != null)
      return index.intValue();
    else
      return -1;
  }  

  public static int registerNamedExtension(String name) {
    Integer index = (Integer)registry.get(name);
    if (index == null) {
      index = new Integer(count++);
      registry.put(name, index);
    }
    return index.intValue();
  }
  
  public void attachNode(Node s) {
    if (server != s) {
      server = s;
      server.attachExtension(this);
    }
  }
  
  public void run() {
    // this extension is driven only via the node
  }
  
  public void setArgs(KeyArgs k) 
    throws Exception
  {
    k.merge(defaults);
     super.setArgs(k);
  }
 
  public Extension() {
    super();
  } 
}

