/* WARNING: This file was automatically generated from ants/InChannel.Java DO NOT EDIT *//*
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries reproduce
 * the notices in supporting documentation, and (3) neither the name of the
 * University nor the Flux Group may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 * 
 * THE UNIVERSITY OF UTAH AND THE FLUX GROUP ALLOW FREE USE OF THIS SOFTWARE
 * IN ITS "AS IS" CONDITION, AND DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY
 * DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * The University requests users to return to flux-dist@cs.utah.edu any
 * improvements that they make and grant the University redistribution rights.
 */
/* 
 *  $Id: InChannel.Java,v 1.9 1998/06/01 22:20:06 gback Exp $
 */

package ants ;
import nodeos .*;

import java.net.*;
import java.io.*;
import java.util.*;

/**
 * The InChannel is responsible for decoding and processing all packets
 * stemming from an incoming flow, denoted by a flow handle.
 * <p>
 * THIS IS THE UNIT OF RESOURCE CONSUMPTION.  One InChannel per protocol
 * is created.  InChannels can be instantiated for both locally installed
 * protocols and those loaded on demand.
 * <p>
 * An InChannel is associated with a flow handle that is its source of
 * packets to decode and process.  Each InChannel will instantiate incoming
 * Capsules in a separate name space.  This is done by providing a separate
 * class loader. 
 * <p>
 * In ANTS 1.2, this used to be ChannelThread.  I'm not really happy with 
 * the name "InChannel", however.  A better name might be "ProtocolProcessor".
 *
 * @author Godmar Back
 *	   David Wetherall
 */

public class InChannel
    extends ManagedObject implements Runnable
{
    // placeholder for soft-state generation identifier
    int cluster;

    // flag saying whether this InChannel was created as the result of
    // a local protocol instantiation or not
    private boolean local;

    // if not local, code groups that need to be loaded - from within the 
    // context of this InChannel.  DLResponse::evaluate will push CodeGroups
    // onto this stack.
    private Stack		codeGroups = new Stack();

    // if local, protocol whose capsule we process
    private Protocol		protocol = null;

    // if not local, class loader for name space
    private DLClassLoader	classLoader;

    // the handle used to received packets
    private FlowHandle	flowHandle;	

    // should this channel automatically resupply buffers to its flow handle?
    private boolean replenish;	

    // has this channel been started yet
    private boolean started;		
    Node client;

    // initializers
    Hashtable initializers = new Hashtable();

    static Hashtable    threadToInChannel = new Hashtable();

    static InChannel currentInChannel()
    {
        InChannel p = (InChannel)threadToInChannel.get(Thread.currentThread());
        Entity.assert(p != null, "Null InChannel");
        return p;
    }

    static void registerIDs(String _cn, TypeID[] ids) {
        currentInChannel().registerIDsInternal(_cn, ids);
    }

    private void registerIDsInternal(String _cn, TypeID[] ids) {
        TypeID[] t = (TypeID[])initializers.get(_cn);
        if (t != null) {
          if (t[0] != ids[0] || t[1] != ids[1] || t[2] != ids[2])
            Entity.error("Different registration for " + _cn);
        }
        initializers.put(_cn, ids);
    }

    byte[] findMID(String name) {
        TypeID[] t = (TypeID[])initializers.get(name);
        if (t == null) Entity.error("No MID registration for "+ name);
        return t[0].unwrap();
    }

    byte[] findGID(String name) {
        TypeID[] t = (TypeID[])initializers.get(name);
        if (t == null) Entity.error("No GID registration for "+ name);
        return t[1].unwrap();
    }

    byte[] findPID(String name) {
        TypeID[] t = (TypeID[])initializers.get(name);
        if (t == null) Entity.error("No PID registration for "+ name);
        return t[2].unwrap();
    }

    /**
     * decode ANTS packets. 
     */
    protected Capsule decode(Buffer buf) throws Exception {

	// the dispatcher stored the method id in the buffer.
	Method m = (Method)buf.getObject();

	if (m == null || m.cl == null)
	    return new DLBootstrapCapsule(m, buf, client.getAddress());

	if (LOGGING)
	    log(L[0], "deserializing "+ m.methodID + " "+ m.cl +
	      " ("+ buf.getLength() +" bytes)");

	return Capsule.create(m.cl, new Xdr(buf));
    }

    /**
     * extract the next capsule from the flow
     */
    public Capsule receive() throws Exception {

	if (LOGGING) log(L[7], "waiting to receive...");
	Buffer b = flowHandle.receive();

	if (b.getLength() > 0) {
	    if (LOGGING) log(L[7], "received " + b.getLength() + " bytes");
	} else {
	    if (LOGGING) log(L[7], "0 byte read???");
	}

	return decode(b);
    }

    /**
     * add a codegroup to this InChannel.
     * This code group will be loaded by that InChannel.
     */
    synchronized void addCodeGroup(CodeGroup g)
    {
	codeGroups.push(g);
    }

    /**
     * Load all codegroups that haven't been loaded yet.
     */
    private void loadCodeGroups(Node n)
    {
	while (!codeGroups.empty()) {
	    String name = getName();
	    CodeGroup codeGroup = (CodeGroup)codeGroups.pop();

	    codeGroup.doactivate(n, flowHandle);
	    setName(name + " " + codeGroup.flowHandle.getName());
	    classLoader.loadOnDemand(codeGroup);
	    refreshDisplay();
	}
    }

    // XXX: move elsewhere
    private void refreshDisplay()
    {
	if (LOGGING) {
	    ConfigurationManager cm = (ConfigurationManager)getBoss();
	    cm.refreshLoggingPanel();
	}
    }

    /**
     * Perform some important initialization.
     * <p>
     * This creates a new process, which currently is only used to 
     * identify the context in which static initializers are executed.
     * It also creates a new namespace for remote protocols.
     */
    void initialize() throws Exception
    {
	// create a new process here
	threadToInChannel.put(Thread.currentThread(), this);

	// this if remote
	if (!local && !codeGroups.empty()) {

	    // create a new name space
	    classLoader = new DLClassLoader();

	    // we don't have a flow handle yet - use the first group's 
	    // FlowHandle. 
	    CodeGroup codeGroup = (CodeGroup)codeGroups.peek();
	    flowHandle = codeGroup.flowHandle;

	    setName("REMOTE: " 
		+ codeGroup.protocolID.toString().substring(0, 8) + ":");

	    loadCodeGroups(client);
	}

	// this if local and we have a non-null protocol
	if (local && protocol != null) {
	    client.registerProtocol(protocol, this, flowHandle);
	}
    }

    /** a one-time start of the channel only */
    public void start() 
    {
	// security
	if (!started) {
	    new Thread(this).start();
	    // wait until initialization complete 
	    synchronized(this) {
		try {
		    wait();
		} catch (InterruptedException ie) {}
	    }
	    started = true;
	}
    }
   
    /** 
     * @return the flow handle associated with that InChannel 
     */
    public FlowHandle getFlowHandle()
    {
	return flowHandle;
    }

    /**
     * constructor for locally instantiated protocols; 
     * protocol and name are known.
     *
     * @param p	The Protocol whose packets should be handled by this InChannel
     */
    public InChannel(Node node, Protocol p, String name, boolean replenish)
    {
	this(node, name, replenish);
	this.protocol = p;
	flowHandle = new EmptyFlowHandle("flow-" + name);
	this.local = true;
    }

    /**
     * constructor for protocols instantiated due to a remote request
     */
    public InChannel(Node node, boolean replenish)
    {
	this(node, "", replenish);
	this.local = false;
	// flowHandle is set later
    }

    // internal code shared between the two constructors goes here
    private InChannel(Node node, String name, boolean replenish)
    {
	super(node.getBoss(), name);

	this.replenish = replenish;
	this.client = node;

	// start out with the same log level as the node?
	setLogLevel(node.getLogLevel());
    }

    /**
     * Main loop for an InChannel.
     * This used to be the main loop of ChannelThread.
     */
    public void run() {
	Capsule c = null;

	try {
	    synchronized (this) {
		initialize();
		notify();
	    }
	} catch (Exception e) {
	    e.printStackTrace();	// XXX
	    return;
	}

	while (true) {
	    try {

		// need to clear this each loop in case receive throws
		c = null;

		// block while we receive and decode from the channel
		c = receive();

		// sources may return null if no capsule needs to be processed
		if (c == null)
		    continue;

		// make it sequential, assume no recursion
		// XXX I think synchronizing on the node is bad idea, which
		// will introduce a huge source for priority inversion
		synchronized (client) {
		    client.context = c;
		    c.evaluate(client);
		}

		// do some postprocessing
		// if the capsule didn't want its buffer saved, recycle it
		if (!c.saveBuffer && c.xdr != null) {
		    flowHandle.addBuffer(c.xdr.getBuffer());
		} else {
		    // during the evaluation of this capsule - which might 
		    // include delivering it to an application, it told us 
		    // that it needed to hold on to the buffer.  Therefore, 
		    // we have to replenish the flow - this allocation goes 
		    // on our budget, of course!
		    if (replenish)
			flowHandle.addBuffer(new Buffer(UDPChannel.BUFSIZE));
		}

		// check periodically whether a new code group must be activated
		if (!local && !codeGroups.empty())
		    loadCodeGroups(client);

	    } catch (Exception e) {
		// log a problem but continue unheeded
		if (Entity.LOGGING)
		    log(Entity.L[0], "terminated " + c + "with " +e);
		e.printStackTrace();
	    }
	}      
    }
}

