/* 
 *  $Id: ManagedObject.Java,v 1.2 1998/05/29 06:00:50 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ANTS_PKG;


/**
 * Shell of all manageable entities.
 *
 * @author David Wetherall
 */


public abstract class ManagedObject
  extends Entity
{

  /** Entity to which we report */
  private Manager boss;
  public Manager getBoss() { return boss; }
  public void setBoss(Manager b) { boss = b; boss.add(this); }
  
  /** Prepared log prefix, kept in sync with name */
  private String lp;
  public void setName(String n) { super.setName(n); lp = n + ": "; }

  /**
   * Report an event to our manager if relevant. Declared as final so
   * it can be inlined and removed at compile-time if logging is off
   */
  
  final protected void log(int level, String msg) {
    if (LOGGING && (logLevel & level) != 0)
      // rely on usage convention to avoid a null pointer exception
      boss.log(level, lp + msg);
  }

  /** No-argument constructor, for calls through newInstance */
  public ManagedObject() {
    super();
  }

  /** Normal constructor gives a name and a manager */
  public ManagedObject(Manager b, String n) {
    super(n);
    setBoss(b);
  }
}


