/* WARNING: This file was automatically generated from ants/Node.Java DO NOT EDIT *//* 
 *  $Id: Node.Java,v 1.11 1998/06/01 05:46:08 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ants ;
import nodeos .*;
import ants.wrapper .*;
import utils .*;

import java.net.*;
import java.util.*;

/**
 * node of an active network
 *
 * @author David Wetherall
 */

public class Node
  extends ManagedObject
  implements Runnable
{
  final public static String[] defaults = { "-updateRoutes", "false" };

  private boolean running;
  private String routefile;
  Capsule context; // accessed by InChannels to set context

  private NodeSecurityManager secure;

  private int address;
  
  public int getAddress() { 
    return address;
  }

  // package level access is used to allow the 
  // routing protocols priveleged access here
  RouteTable routes;

  public RouteTable getRoutes() {
    checkCapsule();
    return routes;
  }

  private Hashtable clients;

  public Hashtable getApplications() {
    checkCapsule();
    return clients;
  }

  // package level access is used to allow the 
  // routing protocols priveleged access here
  Channel link;

  public Channel getChannels() {
    checkCapsule();
    return link;
  }

  private Extension[] extensions;

  public Extension[] getExtensions() {
    checkCapsule();
    return extensions;
  }

  private AutoNodeCache cache;

  public AutoNodeCache getCache() { 
    return cache;
  }
  
  CodeCache capsuleCode;

  /** flow related variables */
  static FlowManager flowManager;
  ANEPDispatcher anepDispatcher;

  // find a InChannel for a given protocol id
  Hashtable protIDToChannel = new Hashtable();	// accessed by ConfigurationMan

  /**
   * find a channel for a given protocol id
   */
  InChannel findInChannel(TypeID protocolID) {
    return (InChannel)protIDToChannel.get(protocolID);
  }

  /**
   * attach an in channel for a given protocol ID
   */
  void attachInChannel(TypeID protocolID, InChannel inChannel) {
    protIDToChannel.put(protocolID, inChannel);
  }

  public void setArgs(KeyArgs k) 
    throws Exception
  {
    k.merge(defaults);

    for (int i = 0; i < k.length(); i++) {
      if (k.key(i).equals("-routes")) {
	routefile = k.arg(i);
	k.strike(i);
      } else if (k.key(i).equals("-updateRoutes")) {
	routes.setUpdates(Boolean.valueOf(k.arg(i)).booleanValue());
	k.strike(i);
      }
    }
    
    super.setArgs(k);
  }

  /**
   * start a built-in protocol
   */
  private FlowHandle startProtocol(Protocol p, String protName) 
      throws Exception 
  {
    boolean replenish = true;
    InChannel inChannel = new InChannel(this, p, protName, replenish);
    FlowHandle protFlow = inChannel.getFlowHandle();
    // XXX
    protFlow.addBuffers(25, ((UDPChannel)link).BUFSIZE);

    attachInChannel(p.protocolID(), inChannel);
    inChannel.start();
    return protFlow;
  }

  public void start() 
    throws Exception 
  {
    Entity.info("Node " + NodeAddress.toString(address) + " starts");

    if (routefile != null) {
      Entity.info("Reading routes from " + routefile);
      routes.read(routefile);
      routes.start();
    }

    // export this node environment to *all* capsule classes
    DLClassLoader.exportClass("ants" +".Node");
    DLClassLoader.exportClass("ants" +".Capsule");
    DLClassLoader.exportClass("ants" +".Xdr");
    DLClassLoader.exportClass("ants" +".NodeAddress");
    DLClassLoader.exportClass("ants" +".ByteArray");
    DLClassLoader.exportClass("ants.wrapper" +".AutoNodeCache");
    DLClassLoader.exportClass("java.lang.Integer");
    DLClassLoader.exportClass("java.lang.StringBuffer");

    // install the system protocols
    startProtocol(new DataProtocol(), "BUILTIN:DataProt");
    startProtocol(new RouteProtocol(), "BUILTIN:RouteProt");

    // register this DL flow handle to receive unknown IDs
    FlowHandle dlFlow = startProtocol(new DLProtocol(), "BUILTIN:DLProt");
    capsuleCode.registerUnknownTypeID(dlFlow);

    // provide for handling of unknown ANEP types
    ANEPChannel anepChannel = new ANEPChannel(this, "ANEP");
    FlowHandle anepFlow  = anepChannel.getFlowHandle();
    anepFlow.addBuffers(10, UDPChannel.BUFSIZE);
    anepDispatcher.registerDefault(anepFlow);
    anepChannel.start();

    // install ourselves as a security mediator
    secure.addMediator(this.getClass());

    // start any node extensions
    for (int i = 0; i < extensions.length; i++) {
      extensions[i].start();
    }

    // give the builtin protocols some time to finish up their initialization
    Thread.sleep(2000);

    // now the node is complete
    running = true;

    // start application and network
    Enumeration e = clients.elements();
    while (e.hasMoreElements()) 
      ((Application)e.nextElement()).start();

    super.start();
  }

  /** Register a protocol for use at this node */
  synchronized void registerProtocol(Protocol p, 
	InChannel inChannel, FlowHandle protFlow) 
    throws Exception
  {
    for (int i = 0; i < p.groups.size(); i++) {
      CodeGroup g = (CodeGroup)p.groups.elementAt(i);

      if (capsuleCode.retrieve(g.groupID) == null) {
	g.pin = true;
	capsuleCode.set(g);
      }
      
      for (int j = 0; j < g.meths.size(); j++) {
	Method m = (Method)g.meths.elementAt(j);

	if (capsuleCode.lookup(m.methodID) == null) {
	  capsuleCode.enter(m);
	}

	m.registerIDs();

	// associate method id with protocol flow
	m.setFlowHandle(protFlow);
      }
      attachInChannel(p.protocolID(), inChannel);
      DLClassLoader.loadForRegister(g, p instanceof BuiltinProtocol);
    }
  }

  /** Register a protocol for use at this node and start a thread to
   * handle incoming packets -- provided for backward compatibility
   */
  public void register(Protocol p) 
    throws Exception
  {
    startProtocol(p, 
	"LOCAL (COMPAT): PID=" + p.protocolID().toString().substring(0,8));
  }

  /**
   * alternate form of register, returns the flow handle associated with
   * that protocol to the caller.
   */
  public FlowHandle registerProtocol(Protocol p, String protName) 
    throws Exception
  {
    return startProtocol(p, "LOCAL:" + protName);
  }

  /** Unregister a protocol previously registered at this node */
  public void unregister(Protocol p) {
    // sorry, just an api call for now
  }

  public void attachApplication(Application c, int port) throws Exception {
    Integer key = new Integer(port);

    if (clients.containsKey(key))
      throw new Exception("Port " + port + " is busy.");
    
    clients.put(key, c);
  }
  
  public void attachChannel(Channel l) {
    if (link != l) {
      link = l;
      link.attachNode(this);
    }
  }

  /** For capsules to quickly retrieve known extensions */
  public Extension getExtension(int index) {
    return extensions[index];
  }

  public void attachExtension(Extension e) {
    int index = Extension.registerNamedExtension(e.getName());

    if (extensions.length < index+1) {
      Extension[] ne = new Extension[index+1];
      System.arraycopy(extensions, 0, ne, 0, extensions.length);
      extensions = ne;
    }

    if (extensions[index] != e){
      extensions[index] = e;
      e.attachNode(this);
    } 
  }

  /** Export a class for use by capsules */
  public void exportClass(String name) throws ClassNotFoundException {
    secure.checkCapsule();
    DLClassLoader.exportClass(name);
  }

  /** Security check to see if a capsule directs the invocation */
  public void checkCapsule() {
    
    // start the check at our caller's site
    secure.checkCapsule(1);
  }

  /** Add a class to the list of security check mediators */
  public void addMediator(Class cl) {
    secure.checkCapsule();
    secure.addMediator(cl);
  }

  public boolean routeForNode(Capsule c, int dst) {
    RouteEntry r = routes.get(dst);
    
    // Log("dst = " + dst);
    
    if (r != null) {
      // Log("routing to " + r.nxt);
      return link.send(c, r.addr);
    } else
      return false;
  }
  
  /**
   * send a capsule to all neighbors.
   * XXX remove that comment if cleared
   * Previously, this method used a list of neighbors, maintained in 
   * RouteTable.  This list, however, included only those neighbors known
   * at initialization time.  djw said that's a bug, so I'm using the
   * current list, returned in an Enumeration by the RouteTable.
   * hope that didn't break the auction.
   */
  public boolean sendToNeighbors(Capsule c) {//ulana
    int i = 0;
    int ok = 0;

    for (Enumeration e = routes.getNeighbors(); e.hasMoreElements(); i++) {
       RouteEntry r = (RouteEntry)e.nextElement();
       if (r != null)
	  if (link.send(c, r.addr) == true) ok++;
    }
    if (ok < i)
      return false;
    else
      return true;
  }

  public boolean deliverToApp(Capsule c, int d) {
    c.saveBuffer = true;
    ((Application)clients.get(new Integer(d))).receive(c);
    return true;
  }


  public long time() {
    return System.currentTimeMillis();
  }

  public void log(String msg) {
    Entity.info(this.context.getClass().getName() + " " + 
		NodeAddress.toString(this.getAddress()) + ": " + msg);
  }

  /** Called by Application base class to send a capsule */ 
  synchronized boolean send(Capsule c) {
    this.context = c;
    return c.evaluate(this);
  }

  /** For use by node components and applications to delay execution */
  public void sleep(long millis) {
    checkCapsule();

    try {
      Thread.sleep(millis);
    } catch (InterruptedException e) {
      ; // who cares
    }
  }
  
  public void run() {
    checkCapsule();
  }


 /** constructs a node with given address */
  public Node(Manager b, String n, String a)
    throws Exception
  {
    super(b, n);

    // first, check system integrity
    secure = new NodeSecurityManager();
    secure.checkCapsule();

    running = false;
    address = NodeAddress.fromString(a);
    routes = new RouteTable(this);
    cache = new AutoNodeCache(this, 1024, 600);
    capsuleCode = new CodeCache(256);
    extensions = new Extension[0];
    clients = new Hashtable();
    if (flowManager == null)
	flowManager = new FlowManager();
    else 
	System.out.println("why are we creating two nodes???");

    anepDispatcher = new ANEPDispatcher();

    // register ANTS dispatcher with ANEP dispatcher
    anepDispatcher.registerANid(ANEPDispatcher.ANTS_ID, capsuleCode);
  }

}
