/* WARNING: This file was automatically generated from ants/NodeAddress.Java DO NOT EDIT *//* 
 *  $Id: NodeAddress.Java,v 1.2 1998/05/29 06:00:52 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ants ;

import java.util.StringTokenizer;
import java.util.NoSuchElementException;


/**
 * Routines for manipulating node addresses.  A node address is 
 * simply a 32 bit integer for efficiency, but these convenience
 * routines exist to convert to and from printed representations
 * and so on.
 *
 * @author David Wetherall
 */

abstract public class NodeAddress {

  final public static boolean multicast(int a) {
    return ((a & 0xF0000000) == 0xE0000000);
  }
  
  public static String toString(int a) {
    String dotted = Integer.toString(a & 0xff);
    dotted = Integer.toString((a >> 8) & 0xff) + "." + dotted;
    dotted = Integer.toString((a >> 16) & 0xff) + "." + dotted;
    dotted = Integer.toString((a >> 24) & 0xff) + "." + dotted;
    return dotted;
  }
  
  public static int fromString(String name)
       throws Exception 
  {
    StringTokenizer st = new StringTokenizer(name, ".");
    if (st.countTokens() != 4)
      throw new Exception("can't parse " + name);
    
    int[] b = new int[4];
    try {
      
      for (int i = 0; i < 4; i++) 
	b[i] = Integer.parseInt(st.nextToken());

    } catch (NumberFormatException e) {

      throw new Exception("non-numeric " + name);

    } catch (NoSuchElementException e) {

      // shouldn't be able to reach here   
      throw new Exception("trouble in fromString!");
    }
    
    return ((b[0]<<24) | (b[1]<<16) | (b[2]<<8) | b[3]); 
  }

  public static void main(String[] args) 
       throws Exception
  {
    String[] h = { "18.31.0.1", "127.0.0.0", "128.0.1.255" };

    for (int i = 0; i < h.length; i++) {
      int n = NodeAddress.fromString(h[i]);
      String s = NodeAddress.toString(n);
      System.out.println(h[i] + " -> " + s);

      if (!h[i].equals(s)) {
	System.out.println("Address MISMATCH");
	System.exit(1);
      }
    }
  }
}
