/* WARNING: This file was automatically generated from ants/NodeCache.Java DO NOT EDIT *//* 
 *  $Id: NodeCache.Java,v 1.2 1998/05/29 06:00:52 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ants ;



/**
 * Entries in the hashtable and lru list.
 * Can be nested under NodeCache in 1.1
 */

class NCElement {
  Object key;
  byte[] family;
  int access;
  Object value;
  NCElement hnext;
  NCElement lnext;
  NCElement lprev;
  int hash;
  int limit;			// discard time in absolute ms
}


/**
 * node cache of managed soft-state
 *
 * combines hashtable and doubly linked list to implement fixed size table
 * with LRU replacement.  all public entry points are synchronized.
 * a background gc thread cleans entries that have expired.
 *
 * @author David Wetherall
 */


public class NodeCache
  implements Runnable
{
  final public static int READSHARED = 1<<0;
  final public static int READEXCLUSIVE = 1<<1;

  // writing isn't used currently
  final public static int WRITESHARED = 1<<2;
  final public static int WRITEEXCLUSIVE = 1<<3;

  final private static int DEFAULTACCESS = READEXCLUSIVE|WRITEEXCLUSIVE;

  private NCElement[] hash;
  private NCElement head;
  private NCElement tail;
  private int capacity;
  private int size;		// number of hash buckets
  private int num;		// current number of entries
  private int duration;		// maximum discard interval
  private Node owner;

  // queue manipulation methods assume e is non-null

  private void movetofront(NCElement e) {
    if (head != e) {

      // now we assume p and head are non-null!

      NCElement n = e.lnext;
      NCElement p = e.lprev;
      p.lnext = n;
      if (n != null) n.lprev = p;
      if (tail == e) tail = p;
      head.lprev = e;
      e.lnext = head;
      e.lprev = null;
      head = e;
    }
  }


  private void insertathead(NCElement e) {
    if (head != null) head.lprev = e;
    e.lnext = head;
    e.lprev = null;
    head = e;
    if (tail == null) tail = e;
    num++;
  }

  
  private void deletefromqueue(NCElement e) {
    NCElement n = e.lnext;
    NCElement p = e.lprev;
    if (p != null) p.lnext = n;
    if (n != null) n.lprev = p;
    if (tail == e) tail = p;
    if (head == e) head = n;
    e.lnext = e.lprev = null;
    num--;
  }

  //
  // remove from both hashtable and queue
  //

  private void delete(NCElement e) {
    int hcode = e.hash;
    int bucket = (hcode & 0x7FFFFFFF) % size;
    NCElement s = hash[bucket];
    NCElement prev = null;

    while (s != null) {
      if (s == e) {
	if (prev != null) prev.hnext = s.hnext;
	else hash[bucket] = s.hnext;
	s.hnext = null;
      }
      prev = s; s = s.hnext;
    }

    deletefromqueue(e);
  }
  
  
  /**
   * get the access flags of an object in the cache by its key
   */

  public synchronized int getAccess(Object key) {
    byte[] family = owner.context.pid();
    int hcode = key.hashCode();
    int bucket = (hcode & 0x7FFFFFFF) % size;

    for (NCElement e = hash[bucket] ; e != null ; e = e.hnext) {
      if ((e.hash == hcode) && (e.family == family) && e.key.equals(key)) {
	return e.access;
      }
    }

    return 0;
  }


  /**
   * set the access flags of an object in the cache by its key
   */

  public synchronized boolean setAccess(Object key, int access) {
    byte[] family = owner.context.pid();
    int hcode = key.hashCode();
    int bucket = (hcode & 0x7FFFFFFF) % size;

    for (NCElement e = hash[bucket] ; e != null ; e = e.hnext) {
      if ((e.hash == hcode) && (e.family == family) && e.key.equals(key)) {

	e.access = access;
	return true;
      }
    }

    return false;
  }


  /**
   * get an object from the cache by its key
   */

  public synchronized Object get(Object key) {
    byte[] family = owner.context.pid();
    int hcode = key.hashCode();
    int bucket = (hcode & 0x7FFFFFFF) % size;
    for (NCElement e = hash[bucket] ; e != null ; e = e.hnext) {
      if ((e.hash == hcode) && (e.family == family) && e.key.equals(key)) {

	// move to head of lru queue
	
	movetofront(e);
	return e.value;
      }
    }
    return null;
  }

  /**
   * get another protocols's object from the cache
   */

  public synchronized Object get(byte[] protocol, Object key) {
    int hcode = key.hashCode();
    int bucket = (hcode & 0x7FFFFFFF) % size;

    for (NCElement e = hash[bucket] ; e != null ; e = e.hnext) {
      if ((e.hash == hcode) && (e.family == protocol) && e.key.equals(key)) {

	if ((e.access & READSHARED) != 0) {

	  // move to head of lru queue
	  
	  movetofront(e);
	  return e.value;

	} else break;
      }
    }
    return null;
  }
  

  /**
   * put an object in the cache by key, with duration and default access
   */

  final public synchronized Object put(Object key, Object value, int time) {
    return put(key, value, time, DEFAULTACCESS);
  }

  /**
   * put an object in the cache by key, with duration and access
   */
  
  public synchronized Object put(Object key, Object value,
				 int time, int access)
  {
    
    if (--this.owner.context.resources <= 0) {
      throw new ResourceLimitException();
    }

    owner.context.saveBuffer = true;

    // bound duration requests
    if (time > duration) time = duration;

    byte[] family = owner.context.pid();
    int hcode = key.hashCode();
    int bucket = (hcode & 0x7FFFFFFF) % size;

    for (NCElement e = hash[bucket] ; e != null ; e = e.hnext) {
      if ((e.hash == hcode) && (e.family == family) && e.key.equals(key)) {

	// overwrite existing value, with new times/access too

	Object p = e.value;
	e.value = value;
	e.access = DEFAULTACCESS;
	e.limit = (int)(owner.time() & 0x7FFFFFFF) + time*1000;
	movetofront(e);

	return p;
      }
    }

    // make a little room; should only fire once

    while (num >= capacity) delete(tail);
    
    NCElement e = new NCElement();
    e.hash = hcode;
    e.key = key;
    e.family = family;
    e.access = DEFAULTACCESS;
    e.value = value;
    e.limit = (int)(owner.time() & 0x7FFFFFFF) + time*1000;
    e.hnext = hash[bucket];
    hash[bucket] = e;

    // add to lru queue

    insertathead(e);
    return null;
  }
  

  /**
   * remove an object from the cache by its key
   */

  public synchronized Object remove(Object key) {
    byte[] family = owner.context.pid();
    int hcode = key.hashCode();
    int bucket = (hcode & 0x7FFFFFFF) % size;
    
    NCElement e = hash[bucket];
    NCElement prev = null;

    while (e != null) {
      if ((e.hash == hcode) && (e.family == family) && e.key.equals(key)) {
	if (prev != null) prev.hnext = e.hnext;
	else hash[bucket] = e.hnext;
	e.hnext = null;

	// remove from lru list, adjust head/tail if necessary
	
	deletefromqueue(e);
	return e.value;
      }
      prev = e; e = e.hnext;
    } 

    return null;
  }
  

  /**
   * clean the cache of expired objects
   */

  public synchronized void clean() {
    int now =  (int)(owner.time() & 0x7FFFFFFF);

    // how to avoid getting caught in gc when there's real work?
    // need to break this up into smaller chunks or change
    // synchronization strategy so we can be pre-empted. (urgh)
    
    NCElement e = tail;
    while (e != null) {
      NCElement n = e.lprev;
      if (e.limit <= now) {

	// should check here for "active" objects, e.g., fusion,
	// that require other cleanup, e.g., stopping a thread ...

	delete(e);
      }
      e = n;
    }
  }
  
  
  /**
   * mainly for testing
   */
  
  public synchronized String toString() {
    StringBuffer buf = new StringBuffer();
    buf.append(num + "/" + capacity + " elements\n");
    
    buf.append("\n" + size + " buckets:\n");
    
    for (int i = 0; i < size; i++) {
      if (hash[i] == null) continue;
      
      buf.append(i + ": ");
      for (NCElement e = hash[i]; e != null; e = e.hnext)
	buf.append(e.key + " ");
      buf.append("\n");
    }
    
    buf.append("\nlru list:\n");
    for (NCElement e = head; e != null; e = e.lnext)
      buf.append(e.key + "\n");
    
    return buf.toString();
  }


  public void run() {
    while (true) {
      clean();
      try {
	owner.sleep(1000);
      } catch (Exception e) {
	; // who cares
      }
    }
  }

  
  public NodeCache(Node owner, int capacity, int duration) {
    
    // arrange to expect only 4/3 entries per bucket
    this.owner = owner;
    this.capacity = capacity;
    size = (int)(capacity*0.75f + 0.5);
    hash = new NCElement[size];    
    for (int i = 0; i < size; i++)
      hash[i] = null;

    head = tail = null;
    this.duration = duration;
    num = 0;

    Thread gc = new Thread(this);
    gc.setDaemon(true);
    gc.setPriority(Thread.MIN_PRIORITY);
    gc.start();
  }


  /**
   * lots of gratuitous testing
   */

  public static void main(String[] args) throws Exception {
    NodeCache nc = new NodeCache(new Node(null, "", ""), 4, 5);

    System.out.println("putting:\n");

    nc.put("one", new Integer(1), 1);
    nc.put("two", new Integer(2), 1);
    nc.put("three", new Integer(3), 1);
    nc.put("four", new Integer(4), 1);
    nc.put("five", new Integer(5), 1);
    nc.put("five", new Integer(5), 1);

    System.out.println(nc);

    System.out.println("fiddling:\n");

    nc.get("three");
    nc.put("six", new Integer(6), 1);
    nc.get("three");
    nc.put("seven", new Integer(7), 1);
    nc.get("three");
    nc.put("eight", new Integer(8), 1);
    nc.get("three");
    nc.put("nine", new Integer(9), 1);
    nc.get("three");
    nc.put("ten", new Integer(10), 1);
    nc.get("three");

    System.out.println(nc);

    System.out.println("removing:\n");

    nc.remove("ten");
    nc.remove("one");

    System.out.println(nc);

    System.out.println("getting:\n");

    Object v;
    if ((v = nc.get("one")) != null) System.out.println(v);
    if ((v = nc.get("two")) != null) System.out.println(v);
    if ((v = nc.get("three")) != null) System.out.println(v);
    if ((v = nc.get("four")) != null) System.out.println(v);
    if ((v = nc.get("five")) != null) System.out.println(v);
    if ((v = nc.get("six")) != null) System.out.println(v);
    if ((v = nc.get("seven")) != null) System.out.println(v);
    if ((v = nc.get("eight")) != null) System.out.println(v);
    if ((v = nc.get("nine")) != null) System.out.println(v);
    if ((v = nc.get("ten")) != null) System.out.println(v);

    System.out.println("sleeping ... ");

    try {
      Thread.sleep(10);
    } catch (Exception e) {
      ; // who cares
    }
   
    System.out.println("left with:\n");
    System.out.println(nc);
  }
  
}



