/* WARNING: This file was automatically generated from ants/NodeSecurityManager.Java DO NOT EDIT *//* 
 *  $Id: NodeSecurityManager.Java,v 1.2 1998/05/29 06:00:53 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ants ;


/**
 * Implements node security checks to constrain capsule access.
 * Access to this class of offered to the Node via package protection.
 *
 * @author David Wetherall */


class NodeSecurityManager
  extends SecurityManager
{
  /** List of classes that may mediate capsule access */
  private Class[] mediators;

  /** Adds a class to the list of classes that mediate capsule access */
  void addMediator(Class cl) {

    // the first thing we do, of course.  allows
    // anyone except capsules to add themselves
    checkCapsule();

    for (int i = 0; i < mediators.length; i++) {
      if (mediators[i] == cl) return;
    }

    Class[] nm = new Class[mediators.length + 1];
    System.arraycopy(mediators, 0, nm, 0, mediators.length);
    nm[mediators.length] = cl;
    mediators = nm;
  }
  

  /** Checks that, if an untrusted capsule call reached here, then the
   * call site at which checkCapsule() was invoked passed through one
   * of the known mediator classes.  Assumes that all untrusted code
   * is classloaded.  */

  void checkCapsule() {
    
    // start the check from our caller
    checkCapsule(1);
  }


  /** Same as checkCapsule(), but begins the search stackbase frames
   * further up the stack */

  void checkCapsule(int stackbase) {

    // break recursion
    if (inCheck) return;

    try {
      inCheck = true;

      // the fast check, for pure node activity
      int unsafe = classLoaderDepth();
      if (unsafe == -1) return;
      
      // the slower, safer check, which shouldn't be on
      // the main path to anywhere.
      
      Class[] cl = getClassContext();
      int safe = cl.length;

      // depth 0 = this method
      // depth 1 = checking method
      // depth 2+ = callers to be checked
      // so we start at 2 plus the requested uplook
      int start = 2 + stackbase;

    search:
      for (int i = start; i < unsafe; i++) {
	for (int j = 0; j < mediators.length; j++) {
	  if (cl[i] == mediators[j]) {
	    safe = i;
	    break search;
	  }
	}
      }
      
      if (safe < unsafe) return;

      Entity.warn("throwing security exception for "+ cl[unsafe].getName());  
      throw new SecurityException("Capsule access not mediated");

    } finally {
      inCheck = false;
    }
  }
  
  /*
  // NOTE: since this security manager is no longer installed on the 
  // runtime (because the cost of invoking its checks is too high --
  // a 1ms setup per datagram send/receive!) the following methods are
  // now redundant.

  // here we subclass all check functions to implement
  // the default policy of checkCapsule
  
  public void checkAccept(String  host, int  port) { checkCapsule(); }
  public void checkAccess(Thread  g) { checkCapsule(); }
  public void checkAccess(ThreadGroup  g) { checkCapsule(); }
  public void checkCreateClassLoader() { checkCapsule(); } 
  public void checkDelete(String  file) { checkCapsule(); }
  public void checkExec(String  cmd) { checkCapsule(); }
  public void checkExit(int  status) { checkCapsule(); }
  public void checkLink(String  lib) { checkCapsule(); }
  public void checkListen(int  port) { checkCapsule(); }
  public void checkPackageAccess(String  pkg) { checkCapsule(); } 
  public void checkPackageDefinition(String  pkg) { checkCapsule(); }
  public void checkPropertiesAccess() { checkCapsule(); }     
  public void checkPropertyAccess(String  key) { checkCapsule(); }
  public void checkRead(java.io.FileDescriptor  fd) { checkCapsule(); } 
  public void checkRead(String  file) { checkCapsule(); }      
  public void checkRead(String  file, Object  context) { checkCapsule(); }
  public void checkSetFactory() { checkCapsule(); }  
  public void checkWrite(java.io.FileDescriptor  fd) { checkCapsule(); }
  public void checkWrite(String  file) { checkCapsule(); } 
  
  public boolean checkTopLevelWindow(Object  window) {
    try {
      checkCapsule(); 
      return true;
    } catch (SecurityException e) {
      return false;
    }
  } 

  // these two just return to make socket send/receive faster
  public void checkConnect(String  host, int  port) { 
    return; 
  }
  
  public void checkConnect(String  host, int  port, Object  context) {
    return;
  }
  */

  /** Construct a fresh node security manager with no restrictions */
  protected NodeSecurityManager() {
    mediators = new Class[0];
  } 
}
