/* 
 *  $Id: Protocol.Java,v 1.5 1998/06/01 05:46:09 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */

package ANTS_PKG;

import java.util.*;


/**
 * Base class for all protocols. Intended to be subclassed
 * for each user-defined protocol for the purpose of collecting
 * capsules into a protocol and registering it at the local node.
 *
 * @author David Wetherall
 */

public class Protocol {
  Vector groups;
  Vector capsules;

  // these protected functions should be used in a 
  // subclass constructor to define a protocol --
  // see "main" for a (static, not constructor) example.

  protected void startProtocolDefn() { }

  protected void endProtocolDefn() {

    // first, need to populate all codegroup groupid lists
    for (int i = 0; i < groups.size(); i++) {
      CodeGroup gi = (CodeGroup)groups.elementAt(i);
      for (int j = 0; j < groups.size(); j++) {
	CodeGroup gj = (CodeGroup)groups.elementAt(j);
	gi.groupIDs.addElement(gj.groupID);
      }
    }

    // now we can compute the protocolid
    // -- this is wasteful, but who cares.
    for (int k = 0; k < groups.size(); k++) {
      CodeGroup gk = (CodeGroup)groups.elementAt(k);
      gk.setProtocolID();
    }

    // and all the methodids
    for (int l = 0; l < capsules.size(); l++) {
      Method m = (Method)capsules.elementAt(l);
      m.setMethodID();
      // registration will be done in protocol process context
    }

    // cl will be set when the method is activated
  }
  
  protected void startGroupDefn() { 
    groups.addElement(new CodeGroup());
  }
  
  protected void endGroupDefn() throws Exception {
    CodeGroup g = (CodeGroup)groups.lastElement();
    g.setGroupID();
    g.setChunks();
  }

  protected void addCapsule(String name) throws Exception {
    CodeGroup g = (CodeGroup)groups.lastElement();
    Method m = new Method(name);    
    m.group = g;
    g.meths.addElement(m);
    g.codeIDs.addElement(m.codeID);
    capsules.addElement(m);
  }

  public String toString() {
    StringBuffer result = new StringBuffer();

    if (groups.size() > 0) {
      result.append(this.getClass().getName() + " "); 
      CodeGroup g = (CodeGroup)groups.elementAt(0);
      result.append(g.protocolID.toString() + "\n");
    }
 
    for (int i = 0; i < groups.size(); i++) {
      CodeGroup g = (CodeGroup)groups.elementAt(i);
      result.append("  CodeGroup " + g.groupID.toString() + "\n");
      
      for (int j = 0; j < g.meths.size(); j++) {
	Method m = (Method)g.meths.elementAt(j);
	result.append("    " + m._cn + " " + m.methodID.toString() + "\n");
      }
    }
    
    return result.toString();
  }

  /**
   * @return protocol id
   */
  TypeID protocolID() {
    return ((CodeGroup)groups.firstElement()).protocolID;
  }

  protected Protocol() {
    groups = new Vector();
    capsules = new Vector();
  }
  
  /**
   * Test with a subclass-like constructor.  The need to create a
   * generic protocol here is the only reason this class isn't abstract.
   */

  public static void main(String args[]) throws Exception {
    if (args.length == 0) {
      Protocol p = new Protocol();
      p.startProtocolDefn();
      
      p.startGroupDefn();
      p.addCapsule("ants.Protocol");
      p.endGroupDefn();
      
      p.startGroupDefn();
      p.addCapsule(UTILS_PKG_S+".MD5");
      p.addCapsule(UTILS_PKG_S+".ClassCode");
      p.endGroupDefn();
      
      p.endProtocolDefn();
    } else {
      Class protocol = Class.forName(args[0]);
      System.out.print(protocol.newInstance());
    }
  }
}

