/* WARNING: This file was automatically generated from ants/RouteUpdateCapsule.Java DO NOT EDIT *//* 
 *  $Id: RouteUpdateCapsule.Java,v 1.4 1998/05/29 06:00:57 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ants ;

import java.util.*;


/**
 * Route update capsule processing, for construction of default routes
 *
 * @author David Wetherall
 */

public class RouteUpdateCapsule extends Capsule {

  // WARNING: this is a builtin capsule, whose version number
  // must be manually kept in synch with  its functionality
  final public static String VERSION = "ants.RouteCapsule 10/24/97";

  final private static byte[] MID = findMID("ants.RouteUpdateCapsule");
  protected byte[] mid() { return MID; }
 
  final private static byte[] PID = findPID("ants.RouteUpdateCapsule");
  protected byte[] pid() { return PID; }

  final static private int ROUTERESOURCES = 64;
 
  private String linkaddr;
  private int nroutes;
  private ByteArray routes;
  
  public int length() {
    int s = Xdr.STRING(linkaddr) + Xdr.INT + Xdr.BYTEARRAY(routes);
    return super.length() + s;
  }
  
  public Xdr encode() {
    Xdr xdr = super.encode();
    xdr.PUT(linkaddr);
    xdr.PUT(nroutes);
    xdr.PUT(routes);
    return xdr;
  }

  public Xdr decode() {
    Xdr xdr = super.decode();
    linkaddr = xdr.STRING();
    nroutes = xdr.INT();
    routes = xdr.BYTEARRAY();
    return xdr;
  }
  
  public boolean evaluate(Node n) {
    if (n.getAddress() ==  getDst()) {
      RouteTable rtab = n.routes;

      // if this node isn't participating, then bail
      if (!rtab.getUpdates()) return true;

      boolean changed = false;
      long now = n.time();

      if (RouteTable.LOGGING) {
	Entity.info("RouteUpdateCapsule: Arrived from "+
		    NodeAddress.toString(getSrc()) + " ("+
		    nroutes + " routes)");
      }

      // (re)synchronize on node to lock out the RouteTable processing
      synchronized (n) {
	RouteEvent n1 = rtab.findFirstTimer();
	
	if (rtab.updateEntry(getSrc(), 0, getSrc(), linkaddr, now))
	  changed = true;
	
	if (nroutes > 0) {
	  Xdr xdr = new Xdr(routes, 0);
	  
	  for (int i = 0; i < nroutes; i++) {
	    int dest = xdr.INT();
	    int dist = xdr.BYTE();
	    
	    if (rtab.updateEntry(dest, dist, getSrc(), linkaddr, now))
	      changed = true;
	  }
	}
	
	if (changed) {
	  // schedule a trigger update if not already pending
	  if (!rtab.getTrigger()) rtab.setTriggerTimer(now);
	}
	
	// wake the scheduler if we've altered the next event
	RouteEvent n2 = rtab.findFirstTimer();
	if (n2.when < n1.when) n.notify();
      }
      
      return true;
      
    } else {
      Entity.info("RouteUpdateCapsule: At "+
		  NodeAddress.toString(n.getAddress()) +
		  " instead of "+
		  NodeAddress.toString(getDst()) +
		  " and very confused");
      return true;
    }
  }  
  
  public RouteUpdateCapsule() { }

  public RouteUpdateCapsule(int src, int dst, ChannelAddress addr, Vector r) {
    this.setDst(dst);
    this.prime(src, ROUTERESOURCES);
    this.linkaddr = addr.toString();
    this.nroutes = r.size();

    if (nroutes > 0) {
      this.routes = new ByteArray(nroutes * (Xdr.INT+Xdr.BYTE));
      Xdr xdr = new Xdr(routes, 0);
      for (Enumeration e = r.elements(); e.hasMoreElements();) {
	RouteEntry u = (RouteEntry)e.nextElement();
	xdr.PUT(u.dst);
	xdr.PUT((byte)u.cost);
      }
    }
  }
  
}
