/* 
 *  $Id: UDPChannel.Java,v 1.6 1998/05/31 23:42:39 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ANTS_PKG;
import NODEOS_PKG.*;

import java.net.*;
import java.io.*;

/**
 * UDP channel for nodes.
 * <p>
 * This channel implements the ANEP protocol.
 *
 * @author David Wetherall
 */

public class UDPChannel
  extends Channel
{
  static final int BUFSIZE = 1500;
  DatagramSocket dsock;
  
  public boolean send(byte[] buf, int length, ChannelAddress dst) {
    if (! (dst instanceof UDPChannelAddress)) {
      Entity.warn("UDPChannel.send() passed non-UDP destination address");
      return false;
    }
    
    UDPChannelAddress a = (UDPChannelAddress)dst;
    DatagramPacket fr = new DatagramPacket(buf, length, a.address, a.port);
    try {
      dsock.send(fr);
      if (LOGGING)
	log(L[7], "transmitted " + fr.getLength() + " bytes");
    } catch (IOException e) {
      // Log("UDPChannel.send datagram send raises " + e);
      return false;
    }
    
    // Log("... done");
    
    return true;
  }
    
  public void attachNode(Node c) {
    super.attachNode(c);

    UDPChannelAddress src = (UDPChannelAddress)this.source;

    // register ANEP dispatcher with UDP source port
    // for now, have the flow manager return the actual socket 
    // so that we can send.
    dsock = Node.flowManager.registerUDP(src.port, c.anepDispatcher);
  }

  public UDPChannel(Manager b, String n, UDPChannelAddress src)
    throws IOException
  {
    super(b, n, src);
    txbuf = new Buffer(BUFSIZE);
  }
  
  public UDPChannel(Manager b, String n, String hp)
    throws Exception
  {
    this(b, n, new UDPChannelAddress(hp));
  }
  
}
