/* 
 *  $Id: Xdr.Java,v 1.5 1998/05/29 20:51:50 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ANTS_PKG;
import NODEOS_PKG.*;

/** 
 * Base class with convenience macros for external data representation.
 * These methods are declared final so that they can be inlined with
 * optimization.
 *
 * @author David Wetherall
 */

public final class Xdr {

  public final static int BOOLEAN = 1;
  public final static int BYTE = 1;
  public final static int SHORT = 2;
  public final static int INT = 4;
  public final static int LONG = 8;
  public final static int TYPEID = 16;

  byte[] buf;
  int offset;
  int len;
  int index = 0;
  Xdr parent;

  public final static int STRING(String x) {
    if (x == null) return BYTE;
    else return BYTE + x.length();
  }

  public final static int BYTEARRAY(ByteArray x) {
    if (x == null) return SHORT;
    else return SHORT + x.len;
  }

  public final int length() { return len; }

  public final void PUT(boolean x) {
    if (index + BOOLEAN > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    if (x) buf[j] = (byte)0xff;
    else buf[j] = (byte)0;
    index += BOOLEAN;
  }

  public final void PUT(byte x) {
    if (index + BYTE > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    buf[j] = x;
    index += BYTE;
  }

  public final void PUT(short x) {
    if (index + SHORT > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    buf[j++] = (byte)(x>>>8);
    buf[j] = (byte)x;
    index += SHORT;
  }

  public final void PUT(int x) {
    if (index + INT > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    buf[j++] = (byte)(x>>>24);
    buf[j++] = (byte)(x>>>16);
    buf[j++] = (byte)(x>>>8);
    buf[j] = (byte)x;
    index += INT;
  }

  public final void PUT(long x) {
    if (index + LONG > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    buf[j++] = (byte)(x>>>56);
    buf[j++] = (byte)(x>>>48);
    buf[j++] = (byte)(x>>>40);
    buf[j++] = (byte)(x>>>32);
    buf[j++] = (byte)(x>>>24);
    buf[j++] = (byte)(x>>>16);
    buf[j++] = (byte)(x>>>8);
    buf[j] = (byte)x;
    index += LONG;
  }

  public final void PUT(TypeID t) {
    if (index + TYPEID > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    System.arraycopy(t.unwrap(), 0, buf, j, TYPEID);
    index += TYPEID;
  }

  public final void PUT(String x) {
    if (index + STRING(x) > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    if (x == null) {
      buf[j] = (byte)0;
      index += BYTE;
    } else {
      int l = x.length();
      buf[j++] = (byte)l;
      byte [] xbytes = x.getBytes();
      System.arraycopy(xbytes, 0, buf, j, l);
      // deprecated: x.getBytes(0, l, buf, j);
      index += BYTE + l;
    }    
  }

  public final void PUT(ByteArray x) {
    if (index + BYTEARRAY(x) > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    if (x == null) {
      buf[j++] = (byte)0; 
      buf[j] = (byte)0;
      index += SHORT;
    } else {
      int l = x.len;
      buf[j++] = (byte)(l>>>8);
      buf[j++] = (byte)l;
      if (x.buf != buf || x.offset != j)
	System.arraycopy(x.buf, x.offset, buf, j, l);
      index += SHORT + l;
    }
  }

  public final void PUT(byte[] x, int start, int length) {
    if (index + length > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    System.arraycopy(x, start, buf, j, length);
    index += length;
  }

  public final boolean BOOLEAN() {
    if (index + BOOLEAN > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    index += BOOLEAN;
    if (buf[j] != 0) return true;
    else return false;
  }

  public final byte BYTE() { 
    if (index + BYTE > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    index += BYTE;
    return (byte)(buf[j]&0xff);
  }
  
  public final short SHORT() {
    if (index + SHORT > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    index += SHORT;
    return (short)((buf[j++]&0xff)<<8 | (buf[j]&0xff));
  }

  public final int INT() {
    if (index + INT > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    index += INT;
    return (buf[j++]&0xff)<<24 | (buf[j++]&0xff)<<16 |
      (buf[j++]&0xff)<<8 | buf[j]&0xff;
  }

  public final long LONG() {
    if (index + LONG > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    index += LONG;
    return ((long)buf[j++]&0xff)<<56 | ((long)buf[j++]&0xff)<<48 |
      ((long)buf[j++]&0xff)<<40 | ((long)buf[j++]&0xff)<<32 |
      ((long)buf[j++]&0xff)<<24 | ((long)buf[j++]&0xff)<<16 |
      ((long)buf[j++]&0xff)<<8 | (long)buf[j]&0xff;
  }

  public final TypeID TYPEID() { 
    if (index + TYPEID > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    index += TYPEID;
    return new TypeID(buf, j);
  }

  public final String STRING() {
    if (index + SHORT > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    int l = buf[j++];
    if (index + BYTE + l > len)
      throw new IndexOutOfBoundsException();
    index += BYTE + l;
    if (l == 0)
      return null;
    else {
      return new String(buf, j, l);
    }
  }

  public final ByteArray BYTEARRAY() { 
    if (index + SHORT > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    int l = (buf[j++]&0xff)<<8 | (buf[j++]&0xff);
    if (index + SHORT + l > len)
      throw new IndexOutOfBoundsException();
    index += SHORT;
    ByteArray data = null;
    if (l != 0) {
      data = new ByteArray(buf, offset + index, l);
    }
    index += l;
    return data;
  }

  public final void BYTES(byte[] x, int start, int length) {
    if (index + length > len)
      throw new IndexOutOfBoundsException();
    int j = offset + index;
    System.arraycopy(buf, j, x, start, length);
    index += length;
  }

  public final void skip(int count) {
    if (index + count > len)
      throw new IndexOutOfBoundsException();
    index += count;
  }

  public Xdr(int len)
  {
    buf = new byte[len];
    offset = 0;
    this.len = len;
  }

  public Xdr(byte[] buf)
  {
    this.buf = buf;
    this.offset = 0;
    this.len = buf.length;
  }

  public Xdr(byte[] buf, int offset)
  {
    this(buf, offset, buf.length-offset);
  }

  public Xdr(byte[] buf, int offset, int len) 
       throws IndexOutOfBoundsException
  {
    if (offset + len > buf.length || offset < 0 || len < 0)
      throw new IndexOutOfBoundsException();
    this.buf = buf;
    this.offset = offset;
    this.len = len;
  }
  
  public Xdr(ByteArray buf, int offset)
  {
    this(buf, offset, buf.len - offset);
  }

  public Xdr(ByteArray buf, int offset, int len)
       throws IndexOutOfBoundsException
  {
    if (offset + len > buf.len || offset < 0 || len < 0)
      throw new IndexOutOfBoundsException();
    this.buf = buf.buf;
    this.offset = buf.offset + offset;
    this.len = len;
  }

  Xdr(Xdr parent, int offset)
  {
    this(parent, offset, parent.len-offset);
  }

  Xdr(Xdr parent, int offset, int len)
       throws IndexOutOfBoundsException
  {
    if (offset < 0 || offset + len > parent.len || len < 0)
      throw new IndexOutOfBoundsException();
    this.parent = parent;
    buf = parent.buf;
    this.offset = parent.offset + offset;
    this.len = len;
  }

  /**
   * additional constructors using nodeos.Buffer
   */
  private Buffer	nodebuf = null;

  /**
   * Return associated nodeos buffer
   */
  Buffer getBuffer()
  {
    return nodebuf;
  }

  Xdr(Buffer nodebuf)
  {
    this(nodebuf, 0, nodebuf.getLength());
  }

  Xdr(Buffer nodebuf, int offset, int length)
  {
    this(nodebuf.getData(), (int)nodebuf.getStart() + offset, length);
    this.nodebuf = nodebuf;
  }

  /**
   * Reset an xdr stream.
   *
   * resets the nodeos buffer, if necessary
   *
   * @return Xdr the Xdr stream that was reset for convenience
   */
  Xdr reset()
  {
    if (nodebuf != null) {
      nodebuf.reset();
      buf = nodebuf.getData();
      offset = (int)nodebuf.getStart();
      len = nodebuf.getLength();
    } else {
      offset = index = 0;
      len = buf.length;
    }
    return this;
  }
}

