/* 
 *  $Id: AutoNodeCache.java,v 1.1.1.1 1998/02/24 04:07:34 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ants.wrapper;

import ants.Node;

/**
 * Node Cache with convenience functions for access
 *
 * The wrapper classes W_*.java and interface glue below should be
 * automatically generated, but aren't yet!  The "type signatures"
 * used to indicate the contents of an object are pseudo-classfile:
 *
 *  I = int, J = long, A = array (since [ isn't a valid in a name) 
 *
 * Names are i1, i2, etc., for the first int, second int, etc.
 * Only type permutations up to length 3 are supported!  The base
 * cases for int and long are covered by java.lang.Integer/Long.
 *
 * @author David Wetherall
 */

public class AutoNodeCache
  extends ants.NodeCache
{
  // interface glue for Integer and Long base cases

  public final synchronized Object get(int i) {
    return super.get(new Integer(i));
  }

  public final synchronized Object put(int i, Object value, int time) {
    return super.put(new Integer(i), value, time);
  }
  
  public final synchronized Object get(long l) {
    return super.get(new Long(l));
  }

  public final synchronized Object put(long l, Object value, int time) {
    return super.put(new Long(l), value, time);
  }  

  // interface glue, one get/put for each wrapper class

  public final synchronized Object get(int i1, long j1) {
    return super.get(new W_IJ(i1, j1));
  }

  public final synchronized Object put(int i1, long j1,
				       Object value, int time)
  {
    return super.put(new W_IJ(i1, j1), value, time);
  }

  public final synchronized Object get(int i1, int i2, long j1) {
    return super.get(new W_IIJ(i1, i2, j1));
  }

  public final synchronized Object put(int i1, int i2, long j1, 
				       Object value, int time)
  {
    return super.put(new W_IIJ(i1, i2, j1), value, time);
  }

  // that's it for this class

  public AutoNodeCache(Node owner, int capacity, int duration) {
    super(owner, capacity, duration);
  }

}



