/* 
 *  $Id: MobileApplication.java,v 1.2 1998/05/28 23:46:44 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package apps;

import java.awt.*;
import java.awt.event.*;

import ants.*;
import utils.*;


class MAUpdate
  implements Runnable 
{
  MobileApplication mapp;

  public void run() {
    int loop = 0;
    while (true) {
      try {
	Thread.sleep(mapp.udelay+10);
      } catch (InterruptedException e) {
	continue;
      }
      
      if (!mapp.away) continue;
      
      mapp.send(new MobileUpdateCapsule(mapp.home,
					mapp.foreign,
					mapp.alternate));
      loop++;
      // mapp.slabel.setText(loop + " updates");
    }
  }
  
  MAUpdate(MobileApplication mapp) {
    this.mapp = mapp;
  }
}

/**
 * primitive Mobile test application
 *
 * @author David Wetherall
 */


public class MobileApplication
  extends Application
  implements ActionListener
{
  Button act;
  Label plabel;
  Label slabel;
  int pcount;
  int mobile;
  int home;
  int foreign;
  int alternate;
  int udelay;
  volatile boolean away;
  
  final public static String[] defaults = { "-update", "1000" };

  synchronized public void receive(Capsule cap) {
    super.receive(cap);
    
    if (cap instanceof MobileCapsule) {
      pcount++;
      plabel.setText(pcount + " packets");
      
      MobileCapsule mcap = (MobileCapsule)cap;
      slabel.setText("via "+ NodeAddress.toString(mcap.next));
    }
  }
  
  public void actionPerformed(ActionEvent evt) {
    if (evt.getSource() == act) {
      if (thisNode().getAddress() == mobile) {
	away = !away;
	plabel.setText(away ? "away" : "home");
      } else
	send(new MobileCapsule(mobile, port, port, null));
    }
  }

  public void setArgs(KeyArgs k) 
    throws Exception
  {
    k.merge(defaults);

    for (int i = 0; i < k.length(); i++) {
      if (k.key(i).equals("-mobile")) {
	mobile = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      } else if (k.key(i).equals("-home")) {
	home = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      } else if (k.key(i).equals("-foreign")) {
	foreign = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      } else if (k.key(i).equals("-alternate")) {
	alternate = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      } else if (k.key(i).equals("-update")) {
	udelay = Integer.parseInt(k.arg(i));
	k.strike(i);
      }
    }

    super.setArgs(k);
  }

  public void start() 
    throws Exception 
  {
    thisNode().register(new MobileProtocol());

    Frame window = FrameCreator.createFrame(getName());
    window.setSize(200, 200);
    
    if (mobile == thisNode().getAddress()) {
      act = new Button("Move");
      window.add("North", act);

      plabel = new Label(away ? "away" : "home", Label.CENTER);
      window.add("Center", plabel);
      
      slabel = new Label("no updates", Label.CENTER);
      window.add("South", slabel);

      new Thread(new MAUpdate(this)).start();
    } else {
      act = new Button("Send to " + NodeAddress.toString(mobile));
      act.addActionListener(this);
      window.add("North", act);
    }

    window.show();
  }

  public MobileApplication() {
    super();
  }
}


