 /* 
  *  $Id: MobileCapsule.java,v 1.3 1998/05/28 23:46:44 gback Exp $
  *  
  *  Copyright 1996 Massachusetts Institute of Technology
  * 
  *  Permission to use, copy, modify, distribute, and sell this software and
  *  its documentation for any purpose is hereby granted without fee, provided
  *  that the above copyright notice appear in all copies and that both that
  *  copyright notice and this permission notice appear in supporting
  *  documentation, and that the name of M.I.T. not be used in advertising or
  *  publicity pertaining to distribution of the software without specific,
  *  written prior permission.  M.I.T. makes no representations about the
  *  suitability of this software for any purpose.  It is provided "as is"
  *  without express or implied warranty.
  * 
  */


package apps;

import ants.*;

 /**
  * ping capsule processing
  *
  * @author David Wetherall
  */

public class MobileCapsule
   extends Capsule
{
  final private static byte[] MID = findMID("apps.MobileCapsule");
  protected byte[] mid() { return MID; }

  final private static byte[] PID = findPID("apps.MobileCapsule");
  protected byte[] pid() { return PID; }

  int next;
  short spt;
  short dpt;
  ByteArray data;

  public int length() {
    int s = Xdr.INT + Xdr.SHORT + Xdr.SHORT + Xdr.BYTEARRAY(data);
    return super.length() + s;
  }

  
  public Xdr encode() {
    Xdr xdr = super.encode();

    xdr.PUT(next);
    xdr.PUT(spt);
    xdr.PUT(dpt);
    xdr.PUT(data);

    return xdr;
  }

  public Xdr decode() {
    Xdr xdr = super.decode();

    next = xdr.INT();
    spt = xdr.SHORT();
    dpt = xdr.SHORT();
    data = xdr.BYTEARRAY();

    return xdr;
  }

  public boolean evaluate(Node n) {
    Integer f = (Integer)n.getCache().get(getDst());
    if (f != null) next = f.intValue();

    if (n.getAddress() != getDst())
      return n.routeForNode(this, next);
    else
      return n.deliverToApp(this, dpt);
  }
  
  public MobileCapsule() { }

  public MobileCapsule(int na, short sa, short da, ByteArray d) {
    setDst(na);
    next = na; // we start to head to original dst
    spt = sa;
    dpt = da;
    data = d;
  }

}
