/* 
 *  $Id: MobileServerApplication.java,v 1.2 1998/05/28 23:46:45 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package apps;

import java.awt.*;
import java.awt.event.*;

import ants.*;
import utils.*;

/**
 * primitive Mobile test application
 *
 * @author David Wetherall
 */


public class MobileServerApplication
  extends Application implements Runnable, ActionListener
{
  Button sender;
  Label output;
  TextField iterations, interval;

  int pcount;
  int mobile;
  int home;
  int foreign;
  int alternate;
  int udelay;
  volatile boolean away;
  
  final public static String[] defaults = { "-update", "1000" };

  synchronized public void receive(Capsule cap) {
    super.receive(cap);
    
    if (cap instanceof MobileUpdateCapsule) 
      System.out.println("What the hell?\n" +
			 "I just received a mobile update capsule");
  }
  

  public void actionPerformed(ActionEvent evt) {
    if (evt.getSource() == sender) {
      new Thread(this).start();
    } 
  }
  

  public void run()
  {
    int iter = Integer.parseInt(iterations.getText());
    double interv = Double.valueOf(interval.getText()).doubleValue();
    long itvMillis = (long) Math.floor(interv), beg, diff;
    int itvNanos = (int) Math.round((interv - Math.floor(interv))*1000000);

    long begTime, endTime;

    for (int i=0; i < iter; i++) {
      beg = System.currentTimeMillis();
      send(new MobileCapsule(mobile, port, port, null));
      if (i%100==99)
	System.out.println("Have sent " + (i+1) + " capsules");
      diff = System.currentTimeMillis() - beg;
      if (itvMillis >= diff) {
	try {
	  Thread.sleep(itvMillis - diff, itvNanos);
	}
	catch (InterruptedException e) {
	  e.printStackTrace(); }
      }
    }
  }

  public void setArgs(KeyArgs k) 
    throws Exception
  {
    k.merge(defaults);

    for (int i = 0; i < k.length(); i++) {
      if (k.key(i).equals("-mobile")) {
	mobile = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      } else if (k.key(i).equals("-home")) {
	home = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      } else if (k.key(i).equals("-foreign")) {
	foreign = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      } else if (k.key(i).equals("-alternate")) {
	alternate = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      } else if (k.key(i).equals("-update")) {
	udelay = Integer.parseInt(k.arg(i));
	k.strike(i);
      }
    }

    super.setArgs(k);
  }

  public void start() 
    throws Exception 
  {
    thisNode().register(new MobileProtocol());

    Frame window = FrameCreator.createFrame(getName());
    window.setSize(400, 200);

    // 2 rows by 2 columns with 2 pixel point spaces in between them
    window.setLayout(new BorderLayout()); 

    sender = new Button("Send to " + NodeAddress.toString(mobile));

    output = new Label("Nothing to report", Label.CENTER);

    Panel iterDisplay = new Panel();
    iterDisplay.setLayout(new BorderLayout());
    iterDisplay.add("Center", 
		    new Label("Num Iterations", Label.CENTER));
    iterations = new TextField("0");
    iterations.setEditable(true);
    iterDisplay.add("South", iterations);

    Panel intervalDisplay = new Panel();
    intervalDisplay.setLayout(new BorderLayout());
    intervalDisplay.add("Center",
			new Label("Send interval (in ms)", Label.CENTER));
    interval = new TextField("0");
    interval.setEditable(true);
    intervalDisplay.add("South", interval);

    Panel topPanel = new Panel(), botPanel = new Panel();
    topPanel.setLayout(new GridLayout(1, 2, 1, 1));
    botPanel.setLayout(new GridLayout(1, 2, 1, 1));

    topPanel.add(sender);
    topPanel.add(output);
    botPanel.add(iterDisplay);
    botPanel.add(intervalDisplay);
    
    window.add("South", botPanel);
    window.add("Center", topPanel);

    window.show();
  }

  public MobileServerApplication() {
    super();
  }
}









