/* 
 *  $Id: MobileUpdateCapsule.java,v 1.3 1998/05/28 23:46:46 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package apps;

import ants.*;

/**
 * multicast subscription capsule processing
 *
 * @author David Wetherall
 */

public class MobileUpdateCapsule
  extends Capsule
{
  final private static byte[] MID = findMID("apps.MobileUpdateCapsule");
  protected byte[] mid() { return MID; }

  final private static byte[] PID = findPID("apps.MobileUpdateCapsule");
  protected byte[] pid() { return PID; }

  int next;
  int forward;

  public int length() {
    return super.length() + Xdr.INT + Xdr.INT;
  }
  
  public Xdr encode() {
    Xdr xdr = super.encode();

    xdr.PUT(next);
    xdr.PUT(forward);

    return xdr;
  }

  public Xdr decode() {
    Xdr xdr = super.decode();

    next = xdr.INT();
    forward = xdr.INT();

    return xdr;
  }

  
  public boolean evaluate(Node n) {
    if (n.getAddress() != next)
      return n.routeForNode(this, next);

    n.getCache().put(getSrc(), new Integer(forward), 5);
    if (n.getAddress() != getDst()) {
      forward = next; next = getDst();
      return n.routeForNode(this, next);
    }

    return true;
  }
  
  public MobileUpdateCapsule() { }

  public MobileUpdateCapsule(int dst, int next, int forward) {
    setDst(dst);
    this.next = next;
    this.forward = forward;
  }

}

