/* 
 *  $Id: MulticastApplication.java,v 1.3 1998/05/28 23:46:46 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package apps;

import java.awt.*;
import java.awt.event.*;

import ants.*;
import utils.*;


class MASubscribe
  implements Runnable 
{
  MulticastApplication mapp;

  public void run() {

    while (true) {
      try {
	Thread.sleep(mapp.sdelay*3/2);
      } catch (InterruptedException e) {
	continue;
      }

      mapp.send(new MulticastSubscribeCapsule(mapp.target,mapp.group));
    }
  }

  MASubscribe(MulticastApplication mapp) {
    this.mapp = mapp;
  }
}

/**
 * primitive multicast test application
 *
 * @author David Wetherall
 */


public class MulticastApplication
  extends Application
  implements ActionListener
{
  Button send;
  Label plabel;
  Label slabel;
  int target;
  int group;
  int sdelay;
  int pcount;
  int scount;
  
  final public static String[] defaults = { "-subscribe", "1000" };

  synchronized public void receive(Capsule cap) {
    super.receive(cap);
    
    if (cap instanceof MulticastCapsule) {
      pcount++;
      plabel.setText(Integer.toString(pcount) + " packets");
    } else if (cap instanceof MulticastSubscribeCapsule) {
      scount++;
      slabel.setText(Integer.toString(scount) + " subscriptions");
    }
  }

  public void actionPerformed(ActionEvent evt) {
    if (evt.getSource() == send) {
      send(new MulticastCapsule(group, target, port, port, null));
    }
  }

  public void setArgs(KeyArgs k) 
    throws Exception
  {
    k.merge(defaults);

    for (int i = 0; i < k.length(); i++) {
      if (k.key(i).equals("-target")) {
	target = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      } else if (k.key(i).equals("-group")) {
	group = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      } else if (k.key(i).equals("-subscribe")) {
	sdelay = Integer.parseInt(k.arg(i));
	k.strike(i);
      }
    }

    super.setArgs(k);
  }

  public void start() 
    throws Exception 
  {
    thisNode().register(new MulticastProtocol());

    Window window = FrameCreator.createFrame(getName());
    window.setSize(200, 200);
    
    window.setName(this.getClass().getName() + "@" + thisNode().getName());
    send = new Button("Send to " + NodeAddress.toString(group));
    send.addActionListener(this);
    window.add("North", send);

    plabel = new Label("no packets", Label.CENTER);
    window.add("Center", plabel);
    
    slabel = new Label("no subscriptions", Label.CENTER);
    window.add("South", slabel);

    if (target != thisNode().getAddress())
      new Thread(new MASubscribe(this)).start();

    window.show();
  }

  public MulticastApplication() {
    super();
  }
}


