 /* 
  *  $Id: MulticastCapsule.java,v 1.4 1998/05/28 23:46:46 gback Exp $
  *  
  *  Copyright 1996 Massachusetts Institute of Technology
  * 
  *  Permission to use, copy, modify, distribute, and sell this software and
  *  its documentation for any purpose is hereby granted without fee, provided
  *  that the above copyright notice appear in all copies and that both that
  *  copyright notice and this permission notice appear in supporting
  *  documentation, and that the name of M.I.T. not be used in advertising or
  *  publicity pertaining to distribution of the software without specific,
  *  written prior permission.  M.I.T. makes no representations about the
  *  suitability of this software for any purpose.  It is provided "as is"
  *  without express or implied warranty.
  * 
  */


package apps;

import ants.*;
import ants.wrapper.*;

/**
 * ping capsule processing
 *
 * @author David Wetherall
 */

public class MulticastCapsule
   extends Capsule
{
  final private static byte[] MID = findMID("apps.MulticastCapsule");
  protected byte[] mid() { return MID; }

  final private static byte[] PID = findPID("apps.MulticastCapsule");
  protected byte[] pid() { return PID; }

  int group;
  int target;
  short spt;
  short dpt;
  ByteArray data;

  public int length() {
    int s = Xdr.INT + Xdr.INT + Xdr.SHORT + Xdr.SHORT + Xdr.BYTEARRAY(data);
    return super.length() + s;
  }

  
  public Xdr encode() {
    Xdr xdr = super.encode();

    xdr.PUT(group);
    xdr.PUT(target);
    xdr.PUT(spt);
    xdr.PUT(dpt);
    xdr.PUT(data);

    return xdr;
  }

  public Xdr decode() {
    Xdr xdr = super.decode();

    group = xdr.INT();
    target = xdr.INT();
    spt = xdr.SHORT();
    dpt = xdr.SHORT();
    data = xdr.BYTEARRAY();

    return xdr;
  }

  public boolean evaluate(Node n) {
    W_JAI m = (W_JAI)n.getCache().get(group, target);
    if (m != null) {
      n.log("forwarding MulticastCapsule at " 
	+ NodeAddress.toString(n.getAddress()));
      if (m.ai1 != null)
	for (int i = 0; i < m.ai1.length; i++)
	  n.routeForNode(this, m.ai1[i]);
      else
	n.deliverToApp(this, dpt);
    }
    return true;
  }
  
  public MulticastCapsule() { }
  
  public MulticastCapsule(int ga, int ta, short sa, short da, ByteArray d) {
    group = ga;
    target = ta;
    spt = sa;
    dpt = da;
    data = d;
  }

}
