/* 
 *  $Id: MulticastClientApplication.java,v 1.1.1.1 1998/02/24 04:07:34 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package apps;

import java.awt.*;

import ants.*;
import utils.*;


class MCASubscribe
  implements Runnable 
{
  MulticastClientApplication mapp;

  public void run() {

    while (true) {
      mapp.send(new MulticastSubscribeCapsule(mapp.target,mapp.group));
      try {
	Thread.sleep(mapp.sdelay+10);
      } catch (InterruptedException e) {}
    }
  }

  MCASubscribe(MulticastClientApplication mapp) {
    this.mapp = mapp;
  }
}

/**
 * primitive multicast test application
 *
 * @author David Wetherall
 */


public class MulticastClientApplication
  extends Application
{
  Label plabel;
  int target, group, sdelay, rTotCount = 0, thruInterval = 100;
  final int oneLess = 99;
  long beginTime;
  
  final public static String[] defaults = { "-subscribe", "10000000" };

  
  synchronized public void receive(Capsule cap) {
    super.receive(cap);

    if (cap instanceof MulticastCapsule) {
      switch ((rTotCount % thruInterval)) {
	//case 0:
	//beginTime = System.currentTimeMillis();
	//break;
      case oneLess:
	double diff = 
	  (double) (System.currentTimeMillis() - beginTime) / 1000.0;
	double thru = ((double) thruInterval) / diff;
	beginTime = System.currentTimeMillis();
	System.out.println("Received the " + (rTotCount + 1) + "th capsule");
	System.out.println("Throughput for last " + thruInterval + 
			   " capsules is " + thru + " caps/sec");
	System.out.flush();
	break;
      }
      
      rTotCount++;
    }
  }
  
  public void setArgs(KeyArgs k) 
    throws Exception
  {
    k.merge(defaults);

    for (int i = 0; i < k.length(); i++) {
      if (k.key(i).equals("-target")) {
	target = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      } else if (k.key(i).equals("-group")) {
	group = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      } else if (k.key(i).equals("-subscribe")) {
	sdelay = Integer.parseInt(k.arg(i));
	k.strike(i);
      }
    }

    super.setArgs(k);
  }

  public void start() 
    throws Exception 
  {
    thisNode().register(new MulticastProtocol());

    new Thread(new MCASubscribe(this)).start();
    beginTime = System.currentTimeMillis();
  }

  public MulticastClientApplication() {
    super();
  }
}


