/* 
 *  $Id: MulticastServerApplication.java,v 1.2 1998/05/28 23:46:47 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package apps;

import java.awt.*;
import java.awt.event.*;

import ants.*;
import utils.*;

/**
 * primitive multicast test application
 *
 * @author David Wetherall
 */


public class MulticastServerApplication
  extends Application implements Runnable, ActionListener
{
  int target, group, sdelay, scount;
  Button sender;
  Label output;
  TextField iterations, interval;
  
  final public static String[] defaults = { "-subscribe", "1000" };

  synchronized public void receive(Capsule cap) {
    super.receive(cap);
    
    System.out.println("WE GOT ONE!");
    if (cap instanceof MulticastSubscribeCapsule) {
      scount++;
      output.setText(Integer.toString(scount) + " subscriptions");
    }
  }
  
  public void actionPerformed(ActionEvent evt) {
    if (evt.getSource() == sender) {
      new Thread(this).start();
    }
  }

  public void run()
  {
    int iter = Integer.parseInt(iterations.getText());
    double interv = Double.valueOf(interval.getText()).doubleValue();
    long itvMillis = (long) Math.floor(interv), beg, diff;
    int itvNanos = (int) Math.round((interv - Math.floor(interv))*1000000);

    long begTime, endTime;

    for (int i=0; i < iter; i++) {
      beg = System.currentTimeMillis();
      send(new MulticastCapsule(group, target, port, port, null));
      if (i%100==99)
	System.out.println("Have sent " + (i+1) + " capsules");
      diff = System.currentTimeMillis() - beg;
      if (itvMillis >= diff) {
	try {
	  Thread.sleep(itvMillis - diff, itvNanos);
	}
	catch (InterruptedException e) {
	  e.printStackTrace(); }
      }
    }
  }

  public void setArgs(KeyArgs k) 
    throws Exception
  {
    k.merge(defaults);

    for (int i = 0; i < k.length(); i++) {
      if (k.key(i).equals("-target")) {
	target = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      } else if (k.key(i).equals("-group")) {
	group = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      } else if (k.key(i).equals("-subscribe")) {
	sdelay = Integer.parseInt(k.arg(i));
	k.strike(i);
      }
    }

    super.setArgs(k);
  }

  public void start() 
    throws Exception 
  {
    thisNode().register(new MulticastProtocol());

    Window window = FrameCreator.createFrame(getName());
    window.setSize(400, 200);
    
    // 2 rows by 2 columns with 2 pixel point spaces in between them
    window.setLayout(new BorderLayout()); 
    
    output = new Label("Nothing to report", Label.CENTER);
    sender = new Button("Send to " + NodeAddress.toString(group));

    Panel iterDisplay = new Panel();
    iterDisplay.setLayout(new BorderLayout());
    iterDisplay.add("Center", 
		    new Label("Num Iterations", Label.CENTER));
    iterations = new TextField("0");
    iterations.setEditable(true);
    iterDisplay.add("South", iterations);

    Panel intervalDisplay = new Panel();
    intervalDisplay.setLayout(new BorderLayout());
    intervalDisplay.add("Center",
			new Label("Send interval (in ms)", Label.CENTER));
    interval = new TextField("0");
    interval.setEditable(true);
    intervalDisplay.add("South", interval);

    Panel topPanel = new Panel(), botPanel = new Panel();
    topPanel.setLayout(new GridLayout(1, 2, 1, 1));
    botPanel.setLayout(new GridLayout(1, 2, 1, 1));

    topPanel.add(sender);
    topPanel.add(output);
    botPanel.add(iterDisplay);
    botPanel.add(intervalDisplay);
    
    window.add("South", botPanel);
    window.add("Center", topPanel);
    window.show();

  }

  public MulticastServerApplication() {
    super();
  }
}




