/* 
 *  $Id: MulticastSubscribeCapsule.java,v 1.4 1998/05/28 23:46:47 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package apps;

import ants.*;
import ants.wrapper.*;

/**
 * multicast subscription capsule processing
 *
 * @author David Wetherall
 */

public class MulticastSubscribeCapsule
  extends Capsule
{
  final private static byte[] MID = findMID("apps.MulticastSubscribeCapsule");
  protected byte[] mid() { return MID; }

  final private static byte[] PID = findPID("apps.MulticastSubscribeCapsule");
  protected byte[] pid() { return PID; }

  int group;
  int target;
  int reverse;

  public int length() {
    return super.length() + Xdr.INT + Xdr.INT + Xdr.INT;
  }
  
  public Xdr encode() {
    Xdr xdr = super.encode();

    xdr.PUT(target);
    xdr.PUT(group);
    xdr.PUT(reverse);

    return xdr;
  }

  public Xdr decode() {
    Xdr xdr = super.decode();

    target = xdr.INT();
    group = xdr.INT();
    reverse = xdr.INT();

    return xdr;
  }

  
  public boolean evaluate(Node n) {
    W_JAI m = (W_JAI)n.getCache().get(group, target);

    if (m == null) {

      // n.log("no info at node "+ NodeAddress.toString(n.getAddress()));

      m = new W_JAI();
      n.getCache().put(group, target, m, 5);
    } else {

      // n.log("found something at node "+ NodeAddress.toString(n.getAddress()));
    
    }

    if (reverse != 0) {
      boolean found = false;
      for (int i = 0; m.ai1 != null && i < m.ai1.length; i++) {
	if (m.ai1[i] == reverse) { found = true; break; }
      }
      if (!found) {
	int len = (m.ai1 != null) ? m.ai1.length : 0;
	int[] nn = new int[len+1];
	if (len > 0) System.arraycopy(m.ai1, 0, nn, 0, len);
	nn[len] = reverse;
	m.ai1 = nn;
      }
    }

    reverse = n.getAddress();
    if (n.time() - m.j1 < 1000) return true;
    else m.j1 = n.time();

    if (n.getAddress() != target) {

      // n.log("onward to "+ NodeAddress.toString(target));

      return n.routeForNode(this, target);
    } // else n.log("delivering to application");

    return n.deliverToApp(this, 0);
  }
  
  public MulticastSubscribeCapsule() { }

  public MulticastSubscribeCapsule(int target, int group) {
    this.target = target;
    this.group = group;
    this.reverse = 0;
  }

}

