package apps;

import java.awt.*;
import java.awt.event.*;

import ants.*;
import utils.*;

/**
 */

public class NullDataServerApplication extends Application 
    implements Runnable, ActionListener
{
  final public static String[] defaults = { "-delay", "100" };
  int target, delay;
  Button sender;
  Label output;
  TextField iterations, interval;
  
  synchronized public void receive(Capsule cap) {
    super.receive(cap);
    output.setText("What the hell?\nI just received a packet.");
  }
 
  public void actionPerformed(ActionEvent evt) {
    if (evt.getSource() == sender) {
      new Thread(this).start();
    }
  }

  public void run()
  {
    int iter = Integer.parseInt(iterations.getText());
    double interv = Double.valueOf(interval.getText()).doubleValue();
    long itvMillis = (long) Math.floor(interv), beg, diff;
    int itvNanos = (int) Math.round((interv - Math.floor(interv))*1000000);

    long begTime, endTime;

    for (int i=0; i < iter; i++) {
      beg = System.currentTimeMillis();
      send(new NullDataCapsule(port, port, target, new ByteArray(1)));

      if (i%100==99)
	System.out.println("Have sent " + (i+1) + " capsules");
      diff = System.currentTimeMillis() - beg;
      if (itvMillis >= diff) {
	try {
	  Thread.sleep(itvMillis - diff, itvNanos);
	}
	catch (InterruptedException e) {
	  e.printStackTrace(); }
      }
    }
  }

  public void setArgs(KeyArgs k) 
    throws Exception
  {
    k.merge(defaults);

    for (int i = 0; i < k.length(); i++) {
      if (k.key(i).equals("-target")) {
	target = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      } else if (k.key(i).equals("-delay")) {
	delay = Integer.parseInt(k.arg(i));
	k.strike(i);
      }
    }

    super.setArgs(k);
  }

  public void start() 
    throws Exception 
  {
    
    thisNode().register(new NullDataProtocol());

    Window window = FrameCreator.createFrame(getName());
    window.setSize(400, 200);

    // 2 rows by 2 columns with 2 pixel point spaces in between them
    window.setLayout(new BorderLayout()); 
    
    sender = new Button("send " + NodeAddress.toString(target));

    output = new Label("Nothing to report", Label.CENTER);

    Panel iterDisplay = new Panel();
    iterDisplay.setLayout(new BorderLayout());
    iterDisplay.add("Center", 
		    new Label("Num Iterations", Label.CENTER));
    iterations = new TextField("0");
    iterations.setEditable(true);
    iterDisplay.add("South", iterations);

    Panel intervalDisplay = new Panel();
    intervalDisplay.setLayout(new BorderLayout());
    intervalDisplay.add("Center",
			new Label("Send interval (in ms)", Label.CENTER));
    interval = new TextField("0");
    interval.setEditable(true);
    intervalDisplay.add("South", interval);

    Panel topPanel = new Panel(), botPanel = new Panel();
    topPanel.setLayout(new GridLayout(1, 2, 1, 1));
    botPanel.setLayout(new GridLayout(1, 2, 1, 1));

    topPanel.add(sender);
    topPanel.add(output);
    botPanel.add(iterDisplay);
    botPanel.add(intervalDisplay);
    
    window.add("South", botPanel);
    window.add("Center", topPanel);
    window.show();
  }

  public NullDataServerApplication() 
    throws Exception 
  {
    super();
  }
}





