/* 
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries reproduce
 * the notices in supporting documentation, and (3) neither the name of the
 * University nor the Flux Group may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 * 
 * THE UNIVERSITY OF UTAH AND THE FLUX GROUP ALLOW FREE USE OF THIS SOFTWARE
 * IN ITS "AS IS" CONDITION, AND DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY
 * DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * The University requests users to return to flux-dist@cs.utah.edu any
 * improvements that they make and grant the University redistribution rights.
 */

/* 
 *  $Id: PingAppNoAWT.java,v 1.4 1998/06/01 22:20:07 gback Exp $
 */
package apps;

import java.awt.*;

import ants.*;
import utils.*;
import nodeos.FlowHandle;

/**
 * version of djw's PingApplication w/o GUI
 *
 * @author Godmar Back
 */

public class PingAppNoAWT
  extends Application
  implements Runnable
{
  final public static String[] defaults = {};
  
  int target, rTotCount = 0, delay;
  long beginTime;
  
  synchronized public void receive(Capsule cap) {
    super.receive(cap);

    log(L[6], " ping capsule delivered to App!");

    switch ((rTotCount % 100)) {
      case 0:
	beginTime = thisNode().time();
	break;
      case 99:
	double diff = 
	  (((double)thisNode().time() - (double) beginTime) / 1000.0);
	double thru = 100.0 / diff;
	System.out.println("Thruput= " + thru + " cap/sec");

	if (cap instanceof PingCapsule) {
	  PingCapsule pcap = (PingCapsule)cap;
	  Xdr buf = new Xdr(pcap.getData(), 0);
	  long lat = thisNode().time() - buf.LONG();
	  System.out.println("Latency= " + lat + " ms");
	}
	System.out.println("Received " + (rTotCount+1) + " packets");
	break;
    }

    rTotCount++;
  }
  
  int iter, interv, nbuf;

  public void run()
  {
    System.out.println("sending " 
	+ iter + " capsules in " 
	+ interv + " ms intervals");

    for (int i=0; i < iter; i++) {
      ByteArray buf = new ByteArray(Xdr.LONG);
      Xdr xdr = new Xdr(buf, 0);
      xdr.PUT(thisNode().time());
      PingCapsule c = new PingCapsule(port, port, target, buf);
      send(c);
      thisNode().sleep(interv);
    }
  }

  public void setArgs(KeyArgs k) 
    throws Exception
  {
    k.merge(defaults);

    for (int i = 0; i < k.length(); i++) {
      if (k.key(i).equals("-target")) {
	target = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      }
      if (k.key(i).equals("-iter")) {
	iter = Integer.parseInt(k.arg(i));
	k.strike(i);
      }
      if (k.key(i).equals("-interv")) {
	interv = Integer.parseInt(k.arg(i));
	k.strike(i);
      }
      if (k.key(i).equals("-nbuf")) {
	nbuf = Integer.parseInt(k.arg(i));
	k.strike(i);
      }
      if (k.key(i).equals("-replenish")) {
	replenish = true;
	k.strike(i);
      }
    }

    super.setArgs(k);
  }

  boolean replenish = false;

  /**
   * start that application. Notice how it's different from
   * PingApplication's start.
   */
  public void start() 
    throws Exception 
  {
    InChannel pingChannel = 
	new InChannel(thisNode(), new PingProtocol(), "PingProtocol", true);

    FlowHandle pingFlow = pingChannel.getFlowHandle();

    // provide initial buffer
    pingFlow.addBuffers(nbuf, 8192);

    // this will start a new thread - note that this thread is started
    // in this application's context, meaning that it will share CPU time
    // with it.
    pingChannel.start();

    new Thread(this).start();
  }

  public PingAppNoAWT() 
    throws Exception 
  {
    super();
  }
}





