
/* 
 *  $Id: PingApplication.java,v 1.3 1998/04/24 06:03:41 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package apps;

import java.awt.*;
import java.awt.event.*;

import ants.*;
import utils.*;

/**
 * test GUI application with ping buttons
 *
 * @author David Wetherall
 */

public class PingApplication
  extends Application
  implements Runnable, ActionListener
{
  final public static String[] defaults = {};
  
  int target, rTotCount = 0, delay;
  long beginTime;
  Button pinger;
  Label latency, received, thruput;
  
  TextField iterations, interval, targethost;
  
  synchronized public void receive(Capsule cap) {
    super.receive(cap);

    switch ((rTotCount % 100)) {
      case 0:
	beginTime = thisNode().time();
	break;
      case 99:
	double diff = 
	  (((double)thisNode().time() - (double) beginTime) / 1000.0);
	double thru = 100.0 / diff;
	thruput.setText(String.valueOf(thru));

	if (cap instanceof PingCapsule) {
	  PingCapsule pcap = (PingCapsule)cap;
	  Xdr buf = new Xdr(pcap.getData(), 0);
	  long lat = thisNode().time() - buf.LONG();
	  latency.setText(Long.toString(lat) + " ms");
	}
	received.setText(Integer.toString(rTotCount + 1));
	break;
    }

    rTotCount++;
  }
  
  public void actionPerformed(ActionEvent evt) {
    if (evt.getSource() == pinger) {
      new Thread(this).start();
    }
  }

  public void run()
  {
    try {
	target = NodeAddress.fromString(targethost.getText());
    } catch (Exception e) {
	target = 0;
    }
    int iter = Integer.parseInt(iterations.getText()),
      interv = Integer.parseInt(interval.getText());

    for (int i=0; i < iter; i++) {
      ByteArray buf = new ByteArray(Xdr.LONG);
      Xdr xdr = new Xdr(buf, 0);
      xdr.PUT(thisNode().time());
      PingCapsule c = new PingCapsule(port, port, target, buf);
      send(c);
      thisNode().sleep(interv);
    }
  }

  public void setArgs(KeyArgs k) 
    throws Exception
  {
    k.merge(defaults);

    for (int i = 0; i < k.length(); i++) {
      if (k.key(i).equals("-target")) {
	target = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      }
    }

    super.setArgs(k);
  }

  public void start() 
    throws Exception 
  {
    thisNode().register(new PingProtocol());

    Frame window = FrameCreator.createFrame(getName());
    window.setSize(400, 200);

    // 2 rows by 2 columns with 2 pixel point spaces in between them
    window.setLayout(new BorderLayout()); 
    
    Panel topleftPanel = new Panel();
    topleftPanel.setLayout(new GridLayout(3, 1, 1, 1));
    topleftPanel.add(pinger = new Button("ping!"));
    pinger.addActionListener(this);

    topleftPanel.add(new Label("Target Host", Label.CENTER));
    targethost = new TextField(NodeAddress.toString(target));
    targethost.setEditable(true);
    topleftPanel.add(targethost);
    
    Panel outputDisplay = new Panel();
    outputDisplay.setLayout(new GridLayout(6, 1, 1, 1));
    outputDisplay.add(new Label("Capsules Received", Label.CENTER));
    received = new Label("0", Label.CENTER);
    outputDisplay.add(received);
    latency = new Label("0", Label.CENTER);
    outputDisplay.add(new Label("Capsule Latency (ms)", Label.CENTER));
    outputDisplay.add(latency);
    thruput = new Label("0", Label.CENTER);
    outputDisplay.add(new Label("Throughput (cap/s)", Label.CENTER));
    outputDisplay.add(thruput);

    Panel iterDisplay = new Panel();
    iterDisplay.setLayout(new BorderLayout());
    iterDisplay.add("Center", 
		    new Label("Num Iterations", Label.CENTER));
    iterations = new TextField("0");
    iterations.setEditable(true);
    iterDisplay.add("South", iterations);

    Panel intervalDisplay = new Panel();
    intervalDisplay.setLayout(new BorderLayout());
    intervalDisplay.add("Center",
			new Label("Ping interval (in ms)", Label.CENTER));
    interval = new TextField("0");
    interval.setEditable(true);
    intervalDisplay.add("South", interval);

    Panel topPanel = new Panel(), botPanel = new Panel();
    topPanel.setLayout(new GridLayout(1, 2, 1, 1));
    botPanel.setLayout(new GridLayout(1, 2, 1, 1));

    topPanel.add(topleftPanel);
    topPanel.add(outputDisplay);
    botPanel.add(iterDisplay);
    botPanel.add(intervalDisplay);
    
    window.add("South", botPanel);
    window.add("Center", topPanel);
    window.pack();
    window.show();
  }

  public PingApplication() 
    throws Exception 
  {
    super();
  }
}





