
/* 
 *  $Id: PingCapsule.java,v 1.3 1998/04/24 06:03:41 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package apps;

import ants.*;


/**
 * ping capsule processing
 *
 * @author David Wetherall
 */

public class PingCapsule
  extends DataCapsule
{
  final private static byte[] MID = findMID("apps.PingCapsule");
  protected byte[] mid() { return MID; }

  final private static byte[] PID = findPID("apps.PingCapsule");
  protected byte[] pid() { return PID; }

  public boolean ping = false;
  
  public int length() {
    return super.length() + Xdr.BOOLEAN;
  }
  
  public Xdr encode() {
    Xdr xdr = super.encode();
    xdr.PUT(ping);

    return xdr;
  }

  public Xdr decode() {
    Xdr xdr = super.decode();
    ping = xdr.BOOLEAN();
 
    return xdr;
  }

  /*
  // dummy to change MD5 checksum
  public boolean change_mid()
  {
    return true;
  }
  */

  public boolean evaluate(Node n) {
    if ( n.getAddress() == getDst()) {
      ping = true;      
    } else if (ping!=true) {
      return n.routeForNode(this, getDst());
    } 
    
    if (n.getAddress() == getSrc()) {      
      return n.deliverToApp(this, dpt);
    } else if (ping) {
      return n.routeForNode(this, getSrc());
    }

    return false; 
  }
  
  public PingCapsule() { }

  public PingCapsule(short sa, short da, int na, ByteArray d) {
    super(sa, da, na, d);
  }

}

