/* 
 *  $Id: AuctionAckCapsule.java,v 1.1.1.1 1998/02/25 00:06:15 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package auction;

import ants.*;

/**
 * ack capsule processing for auction
 *
 * @author Ulana Legedza
 */

public class AuctionAckCapsule extends DataCapsule {

  final private static byte[] MID = findMID("auction.AuctionAckCapsule");
  final protected byte[] mid() { return MID; }

  final private static byte[] PID = findPID("auction.AuctionAckCapsule");
  final protected byte[] pid() { return PID; }

  public int client_id;
  public int sequence_num;
  boolean accepted = false;
  int current_price;
    
  public int length() {
    return super.length() + Xdr.INT + Xdr.INT + 
      Xdr.BOOLEAN + Xdr.INT;
  }
  
  public Xdr encode() {
    Xdr xdr = super.encode();
    xdr.PUT(client_id);
    xdr.PUT(sequence_num);
    xdr.PUT(accepted);
    xdr.PUT(current_price);
    
    return xdr;
  }

  public Xdr decode() {
    Xdr xdr = super.decode();
    client_id = xdr.INT(); 
    sequence_num = xdr.INT();
    accepted = xdr.BOOLEAN();
    current_price = xdr.INT();
     
    return xdr;
  }

  
  public boolean evaluate(Node n) {
 
    n.log("evaluating ack capsule " +
	  NodeAddress.toString(this.getDst()) + " \n");
 
    if (n.getAddress() == getDst())
      return n.deliverToApp(this, dpt);
    else 
      return n.routeForNode(this, getDst());
      
  }

  public AuctionAckCapsule() { }
  
  public AuctionAckCapsule(int sequence_num, int client_id, 
			   boolean accepted, int price){

    this.sequence_num = sequence_num;
    this.client_id = client_id;
    this.accepted = accepted;
    this.current_price = price;
  }


  public AuctionAckCapsule(int sequence_num, int client_id, 
			   boolean accepted, int price, 
			   short sa, short da, int na, ByteArray d){
    super(sa, da, na, d);

    this.sequence_num = sequence_num;
    this.client_id = client_id;
    this.accepted = accepted;
    this.current_price = price;
    
  }

}

