/* 
 *  $Id: AuctionBidCapsule.java,v 1.1.1.1 1998/02/25 00:06:15 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package auction;

import ants.*;

/**
 * bid capsule processing for auction
 *
 * @author Ulana Legedza
 */

public class AuctionBidCapsule
  extends DataCapsule
{
  final private static byte[] MID = findMID("auction.AuctionBidCapsule");
  protected byte[] mid() { return MID; }

  final private static byte[] PID = findPID("auction.AuctionBidCapsule");
  protected byte[] pid() { return PID; }

  public int client_id;
  public int bid;
  public int sequence_num;
  
  
  public int length() {
    return super.length() + Xdr.INT + Xdr.INT + Xdr.INT;
  }
  
  public Xdr encode() {
    Xdr xdr = super.encode();
    xdr.PUT(client_id);
    xdr.PUT(bid);
    xdr.PUT(sequence_num);
   
    return xdr;
  }

  public Xdr decode() {
    Xdr xdr = super.decode();
    client_id = xdr.INT();
    bid = xdr.INT();
    sequence_num = xdr.INT();
    
    return xdr;
  }

  
  public boolean evaluate(Node n) {
   
    n.log("evaluating auction capsule \n");

    Object cached_object = n.getCache().get(99);
    if (cached_object != null) {
      
      n.log("object not null \n");

      int cached_price = ((Integer)cached_object).intValue();
      if (cached_price > bid) {  //not <=, b/c what if retransmission
	
	n.log("turning back a bid " + bid + " \n");
	
	AuctionAckCapsule ack = 
	  new AuctionAckCapsule(sequence_num, client_id, false, cached_price, 
				dpt, spt, getSrc(), null);

	ack.prime(this);

	return n.routeForNode(ack, ack.getDst());
      }
    }

   

    if (n.getAddress() == getDst())
      return n.deliverToApp(this, dpt);
    else 
      return n.routeForNode(this, getDst());
    
  }


  public AuctionBidCapsule() { }
  
  public AuctionBidCapsule(int bid, int client_id, int sequence_num) {
    this.bid = bid;
    this.client_id = client_id;
    this.sequence_num = sequence_num;
  }


  public AuctionBidCapsule(int bid, int client_id, int sequence_num,
			   short sa, short da, int na, ByteArray d) {

    super(sa, da, na, d);

    this.bid = bid;
    this.client_id = client_id;
    this.sequence_num = sequence_num;
  }

}

