/* 
 *  $Id: AuctionClientApplication.java,v 1.2 1998/05/28 23:47:17 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package auction;

import java.awt.*;
import java.awt.event.*;

import ants.*;
import utils.*;

/**
 * Auction Client simulating up to 100 bidders
 *
 * @author Ulana Legedza
 */

public class AuctionClientApplication extends Application 
    implements Runnable, ActionListener
{
  int num_clients = 0;

  int[] current_price = new int[100];

  int[] bid = new int[100];
  int[] sequence = new int[100];
  boolean[] winning = new boolean[100]; 
  Integer[] locks = new Integer[100];
  
  long[] timeouts = new long[100];
  long timeout_value = 6000; //6 seconds
  int[] tries = new int[100];
  
  int target = 0;
  int group_id = -1;


  Button bidder;
  Label price, buyer, thruput;
  
  TextField clients, interval;

  
  synchronized public void receive(Capsule cap) {
    boolean need_to_send = false;
    AuctionBidCapsule c;

    super.receive(cap);

    if (cap instanceof AuctionAckCapsule) {
      AuctionAckCapsule ack = (AuctionAckCapsule)cap;

      int client_index = ack.client_id - group_id;

      if (ack.sequence_num < sequence[client_index])
	return; //duplicate ack

      synchronized (locks[client_index]) {
	timeouts[client_index] = 0; //turn off timer

	current_price[client_index] = ack.current_price;
	sequence[client_index]++;

	//System.out.println("receiving acknowledgment \n");

	if (ack.accepted == true || 
	    (ack.current_price == bid[client_index] &&
	     winning[client_index] == true)) { //just keep checking price

	  winning[client_index] = true;
   
	  c = new AuctionBidCapsule(ack.current_price, ack.client_id, 
				    sequence[client_index],
				    port, port, target, null);
	  thisNode().log("checking \n");
	  need_to_send = true;
	  
	} else {
	  
	  winning[client_index] = false;
	  
	  bid[client_index] = ack.current_price + 
	    (int)(java.lang.Math.random() * 100);
	  
	  c = new AuctionBidCapsule(bid[client_index], ack.client_id,
				    sequence[client_index], 
				    port, port, target, null);
	  
	  thisNode().log("re bidding \n");
	  need_to_send = true;		  
	}
	
	tries[client_index] = 0;
	timeouts[client_index] = System.currentTimeMillis() + timeout_value;
	System.out.println(Long.toString(timeouts[client_index]));

	price.setText(Integer.toString(current_price[client_index]));
      }
      if (need_to_send == true)
	send(c);
    }
  }

  public void actionPerformed(ActionEvent evt) {
    if (evt.getSource() == bidder) {
      new Thread(this).start();
    }
  } 
  
  public void run() {

    int i;
    num_clients = Integer.parseInt(clients.getText());

    Thread this_thread = Thread.currentThread();

    for (i = 0; i < num_clients; i++) {

      current_price[i] = 0;
      tries[i] = 0;
      sequence[i] = 0;
      winning[i] = false;

      bid[i] = current_price[i] + (int)(java.lang.Math.random()*100);

      locks[i] = new Integer(0);

      AuctionBidCapsule c = 
	new AuctionBidCapsule(bid[i], group_id + i, sequence[i],
			      port, port, target, null);
      send(c);

      timeouts[i] = System.currentTimeMillis() + timeout_value;
      System.out.println(Long.toString(timeouts[i]));
    }
    
   
    while (true) {
      try {
	this_thread.sleep((long)500);
      } catch (Exception e) {}
      
      //System.out.println("about to check timers \n");
      
      check_timers();

      //System.out.println("finished checking timers \n");
    }
    
  }

  public void setArgs(KeyArgs k) 
    throws Exception
  {
    for (int i = 0; i < k.length(); i++) {
      if (k.key(i).equals("-target")) {
	target = NodeAddress.fromString(k.arg(i));
	k.strike(i);
      } else if (k.key(i).equals("-clientid")) {
	group_id = Integer.parseInt(k.arg(i));
	k.strike(i);
      }
    }

    super.setArgs(k);
  }

  public void start() 
    throws Exception {

    thisNode().register(new AuctionProtocol());      

    Window window = FrameCreator.createFrame(getName());
    window.setSize(350, 200);

    // 2 rows by 2 columns with 2 pixel point spaces in between them
    window.setLayout(new BorderLayout()); 
    
    bidder = new Button("start bidding (after starting server)");
    bidder.addActionListener(this);
    
    Panel outputDisplay = new Panel();
    outputDisplay.setLayout(new GridLayout(2, 1, 1, 1));

    outputDisplay.add(new Label("Current Price", Label.CENTER));
    price = new Label("0", Label.CENTER);
    outputDisplay.add(price);

    Panel clientsDisplay = new Panel();
    clientsDisplay.setLayout(new BorderLayout());
    clientsDisplay.add("Center", 
		    new Label("Number of Clients", Label.CENTER));
    clients = new TextField("5");
    clients.setEditable(true);
    clientsDisplay.add("South", clients);

    Panel topPanel = new Panel(), botPanel = new Panel();
    topPanel.setLayout(new GridLayout(2, 1, 1, 1));
    botPanel.setLayout(new GridLayout(1, 1, 1, 1));

    topPanel.add(bidder);
    topPanel.add(outputDisplay);
    botPanel.add(clientsDisplay);
    
    window.add("South", botPanel);
    window.add("Center", topPanel);
    window.show();
  }

  public AuctionClientApplication() 
    throws Exception 
  {
    super();
  }

  


  private void check_timers() {
    int i;
    long current_time = System.currentTimeMillis();
    AuctionBidCapsule c = new AuctionBidCapsule(); //init to satisfy compiler
    boolean need_to_send;
        
    for (i = 0; i < num_clients; i++) {
      need_to_send = false;

      synchronized (locks[i]) {
	if (timeouts[i] <= current_time /*&& timeouts[i] != 0 */) {
	  need_to_send = true;
	  tries[i]++;
	  c = new AuctionBidCapsule(bid[i], group_id + i, sequence[i], 
				    port, port, target, null);
	
	  //System.out.println("sending retransmission" + bid[i] + " "  + sequence[i] + " "  + tries[i] + " " + i + " \n");

	  timeouts[i] = current_time + timeout_value;
	}
      } 
      if (need_to_send == true)
	send(c);

      //System.out.println("sent retransmission" + bid[i] + " "  + sequence[i] + " "  + tries[i] + " " + i + " \n");
    }
  }
}





