/* 
 *  $Id: AuctionFilterCapsule.java,v 1.1.1.1 1998/02/25 00:06:15 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package auction;

import ants.*;

/**
 * capsule for updating auction filter
 *
 * @author Ulana Legedza
 */

public class AuctionFilterCapsule
  extends DataCapsule
{
  final private static byte[] MID = findMID("auction.AuctionFilterCapsule");
  protected byte[] mid() { return MID; }

  final private static byte[] PID = findPID("auction.AuctionFilterCapsule");
  protected byte[] pid() { return PID; }

  int current_price;
  int hop_distance;
  
  
  public int length() {
    return super.length() + Xdr.INT + Xdr.INT;
  }
  
  public Xdr encode() {
    Xdr xdr = super.encode();
    xdr.PUT(current_price);
    xdr.PUT(hop_distance);

    return xdr;
  }

  public Xdr decode() {
    Xdr xdr = super.decode();
    current_price = xdr.INT();
    hop_distance = xdr.INT(); 
 
    return xdr;
  }

  
  synchronized public boolean evaluate(Node n) {
   
    int old_price;

    n.log("installing filter " + current_price + "\n");

    Object value = n.getCache().get(99);
    if (value != null) {
      old_price = ((Integer) value).intValue();
    } else {
      old_price = 0;
    }

    if (old_price < current_price)
      n.getCache().put(99, new Integer(current_price), 5);
    hop_distance--;


    if (hop_distance > 0 && old_price < current_price)
      return n.sendToNeighbors(this);
    else
      return true;
  }

  public AuctionFilterCapsule() { }
  
  public AuctionFilterCapsule(int price, int distance){
    this.current_price = price;
    this.hop_distance = distance;
  }


  public AuctionFilterCapsule(int price, int distance,
			      short sa, short da, 
			      int na, ByteArray d) {

    super(sa, da, na, d);

    this.current_price = price;
    this.hop_distance = distance;
  }

}

