/* 
 *  $Id: AuctionServerApplication.java,v 1.2 1998/05/28 23:47:17 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package auction;

import java.awt.*;
import java.awt.event.*;

import ants.*;
import utils.*;

/**
 * auction server
 *
 * @author Ulana Legedza
 */

public class AuctionServerApplication extends Application 
    implements Runnable, ActionListener
{

  int count = 0;

  boolean filter = false; //should be same value as in start()

  int current_price = 0;
  int current_buyer = -1;

  int target = 0;
  int client_id = -1;

  int num_winners = 0;
  int num_losers = 0;

  Button starter, toggle;
  Label price, buyer, win_percent, filtering;
  
  TextField dummy;

  
  synchronized public void receive(Capsule cap) {
    super.receive(cap);

    if (cap instanceof AuctionBidCapsule) {
      AuctionBidCapsule auction_cap = (AuctionBidCapsule)cap;

      if (++count % 100 == 13) //TEST: PACKET DROPS
	return;

      AuctionAckCapsule ack = 
	new AuctionAckCapsule(auction_cap.sequence_num, auction_cap.client_id,
			      false, 0, 
			      auction_cap.dpt, auction_cap.spt, 
			      auction_cap.getSrc(), null);
      
      if (auction_cap.bid > current_price ||
	  ((auction_cap.bid == current_price) &&
	   (auction_cap.client_id == current_buyer))) {
	current_price = auction_cap.bid;
	current_buyer = auction_cap.client_id;
	ack.accepted = true;
	num_winners++;
      } else {
	ack.accepted = false;
	num_losers++;
      }

      ack.current_price = current_price;
      
           
      price.setText(Integer.toString(current_price));
      buyer.setText(Integer.toString(current_buyer));
      win_percent.setText(Integer.toString((int)( ( ((float)num_winners)/ 
						   ((float)(num_losers + 
							    num_winners))) 
						 *100)));
      
      //System.out.println("sending ack" + NodeAddress.toString(ack.getDst()) + "\n");

      send(ack);  
        
      AuctionFilterCapsule filter_cap = 
	new AuctionFilterCapsule(current_price, 2, 
				 port, port, 0, null);

      //System.out.println("sending filter \n");

      if (filter == true)
	send(filter_cap);

      //System.out.println("sent filter \n");

    }
  }
  
  public void actionPerformed(ActionEvent evt) {
    if (evt.getSource() == starter) {
      new Thread(this).start();
    } else if (evt.getSource() == toggle) {
      if (filter == true) {
	filter = false;
	filtering.setText("false");
      } else {
	filter = true;
	filtering.setText("true");
      }
    }
  } 

  public void run() {

   ;
  }

  public void setArgs(KeyArgs k) 
    throws Exception
  {
    super.setArgs(k);
  }

  public void start() 
    throws Exception {
      
    thisNode().register(new AuctionProtocol());      

    Window window = FrameCreator.createFrame(getName());
    window.setSize(400, 300);

    // 2 rows by 2 columns with 2 pixel point spaces in between them
    window.setLayout(new BorderLayout()); 
    
    starter = new Button("start server");
    starter.addActionListener(this);
    toggle = new Button("toggle bid filtering on/off");
    toggle.addActionListener(this);

    Panel outputDisplay = new Panel();
    outputDisplay.setLayout(new GridLayout(6, 1, 1, 1));

    outputDisplay.add(new Label("Current Price", Label.CENTER));
    price = new Label("0", Label.CENTER);
    outputDisplay.add(price);

    outputDisplay.add(new Label("Current Buyer", Label.CENTER));
    buyer = new Label("0", Label.CENTER);
    outputDisplay.add(buyer);

    outputDisplay.add(new Label("percent winners", Label.CENTER));
    win_percent = new Label("0", Label.CENTER);
    outputDisplay.add(win_percent);

    Panel toggleDisplay = new Panel();
    toggleDisplay.setLayout(new GridLayout(2, 1, 1, 1));

    toggleDisplay.add(new Label("Filtering On", Label.CENTER));
    filtering = new Label("false", Label.CENTER);
    toggleDisplay.add(filtering);

    Panel topPanel = new Panel();
    topPanel.setLayout(new GridLayout(2, 2, 1, 1));

    topPanel.add(starter);
    topPanel.add(outputDisplay);
    topPanel.add(toggle);
    topPanel.add(toggleDisplay);
    
    window.add("Center", topPanel);
    window.show();
  }

  public AuctionServerApplication() 
    throws Exception 
  {
    super();
  }
}





