/* 
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries reproduce
 * the notices in supporting documentation, and (3) neither the name of the
 * University nor the Flux Group may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 * 
 * THE UNIVERSITY OF UTAH AND THE FLUX GROUP ALLOW FREE USE OF THIS SOFTWARE
 * IN ITS "AS IS" CONDITION, AND DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY
 * DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * The University requests users to return to flux-dist@cs.utah.edu any
 * improvements that they make and grant the University redistribution rights.
 */

package NODEOS_PKG;

/**
 * Flow handle that checks one word, in BIG ENDIAN order, against a
 * known word with which the handle was initialized.
 */
public class CheckOneWordFlowHandle extends FlowHandle
{
    private int     id;
    private byte    idb[];

    /**
     * Class constructor.
     *
     * @param id	identification word
     * @param name	human-readable name
     */
    public CheckOneWordFlowHandle(int id, String name)
    {
	super(name);
	this.id = id;
	this.idb = new byte[4];
	for (int i = 0; i < 4; i++)
	    idb[i] = (byte)(id >> ((3-i) * 8));
    }

    /**
     * Checks whether that buffer belongs to that flow.
     */
    public boolean checkBuffer(Buffer buf, int offset)
    {
	byte b[] = buf.getData();

	for (int i = 0; i < 4; i++)
	    if (idb[i] != b[offset+i])
		return false;

	lastHeaderLength = 4;
	return true;
    }
}

