/* 
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries reproduce
 * the notices in supporting documentation, and (3) neither the name of the
 * University nor the Flux Group may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 * 
 * THE UNIVERSITY OF UTAH AND THE FLUX GROUP ALLOW FREE USE OF THIS SOFTWARE
 * IN ITS "AS IS" CONDITION, AND DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY
 * DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * The University requests users to return to flux-dist@cs.utah.edu any
 * improvements that they make and grant the University redistribution rights.
 */

package NODEOS_PKG;

/**
 * An empty flow handle. 
 * <p>
 * This class completes the abstract implementation of a flow handle
 * by providing a checkBuffer method which always returns true.
 * <p>
 * The purpose of this class is if packets are to be dispatched to
 * a flow unconditionally.  So the "Empty" in the name refers to an
 * empty checkBuffer method.
 * <p>
 * This flow handle can be named for debugging purposes.
 */
public class EmptyFlowHandle extends FlowHandle
{
    /**
     * Class constructor.
     *
     * @param name	a human-readable name
     */
    public EmptyFlowHandle(String name) 
    {
	super(name);
	checkAlwaysTrue = true;
	lastHeaderLength = 0;
    }

    /**
     * Check whether that buffer belongs to that flow.
     *
     * @return always true
     */
    public boolean checkBuffer(Buffer b, int offset)
    {
	return true;
    }
}

