/* 
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries reproduce
 * the notices in supporting documentation, and (3) neither the name of the
 * University nor the Flux Group may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 * 
 * THE UNIVERSITY OF UTAH AND THE FLUX GROUP ALLOW FREE USE OF THIS SOFTWARE
 * IN ITS "AS IS" CONDITION, AND DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY
 * DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * The University requests users to return to flux-dist@cs.utah.edu any
 * improvements that they make and grant the University redistribution rights.
 */

package NODEOS_PKG;

/**
 * A flow dispatcher is an interface that can be used as a generic
 * receiver for packets contained in buffers.  
 *
 * @author 	Godmar Back
 *
 * @see	Buffer
 * @see	FlowHandle
 * @see	SimpleFlowDispatcher
 */
public interface FlowDispatcher
{
    /**
     * Dispatch a given buffer.
     *
     * @param b           buffer to be dispatched.
     * @param offset      offset in buffer, ignoring the buffer's start offset.
     * 
     * @return a free buffer to be exchanged with b. Returns null otherwise.
     *		If the exchange flags of b is not set, null must be returned.
     *
     * @see Buffer
     */
    Buffer dispatchBuffer(Buffer b, int offset);
}
