/* 
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries reproduce
 * the notices in supporting documentation, and (3) neither the name of the
 * University nor the Flux Group may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 * 
 * THE UNIVERSITY OF UTAH AND THE FLUX GROUP ALLOW FREE USE OF THIS SOFTWARE
 * IN ITS "AS IS" CONDITION, AND DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY
 * DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * The University requests users to return to flux-dist@cs.utah.edu any
 * improvements that they make and grant the University redistribution rights.
 */

package NODEOS_PKG;

/**
 * A simple doubly linked list implementation.
 * <p>
 * This is really straightforward.
 *
 * @author Godmar Back
 *
 * @see Linkable
 */
public class LinkedList {
    /**
     * reference to first and last list element.
     */
    /* package */Linkable	first, last;

    /**
     * Check whether list is empty.
     *
     * @return	true if list is empty
     */
    public boolean isEmpty() {
	return first == null;
    }

    /**
     * Counts the number of elements in the list.
     *
     * @return	number of elements in the list.
     */
    public int count()
    {
	int c = 0;
	for (Linkable f = first; f != null; f = f.next)
	    c++;
	return c;
    }

    /**
     * Appends a Linkable to this list.
     *
     * @param l	A Linkable to be appended.
     */ 
    public void append(Linkable l)
    {
        if (first == null) {
            l.next = l.prev = null;
            first = last = l; 
        } else {
            l.next = null;
            l.prev = last; 
            last.next = l;
            last = l;
        }
    }       

    /**
     * Removes a Linkable from this list.
     *
     * @param l Linkable to be removed.
     */
    public void remove(Linkable l) 
    {
        if (l.next != null)  
            l.next.prev = l.prev;
        else
            last = l.prev;
        
        if (l.prev != null)
            l.prev.next = l.next;
        else
            first = l.next;
    }   
}
