/* WARNING: This file was automatically generated from nodeos/SimpleFlowDispatcher.Java DO NOT EDIT *//* 
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries reproduce
 * the notices in supporting documentation, and (3) neither the name of the
 * University nor the Flux Group may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 * 
 * THE UNIVERSITY OF UTAH AND THE FLUX GROUP ALLOW FREE USE OF THIS SOFTWARE
 * IN ITS "AS IS" CONDITION, AND DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY
 * DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * The University requests users to return to flux-dist@cs.utah.edu any
 * improvements that they make and grant the University redistribution rights.
 */

package nodeos ;

import java.net.*;
import java.io.IOException;

/**
 * A simple flow dispatcher.
 *
 * This dispatcher maintains a list of flow dispatchers. 
 * Upon dispatch, it will simply pass the buffer on to every member of that
 * list. If the list contains flow handles, it will only dispatch the buffers
 * to them after invoking checkBuffer, which identifies a buffer as belonging
 * to a certain flow.
 *
 * @author Godmar Back
 *
 * @see FlowDispatcher
 * @see FlowHandle
 */
public class SimpleFlowDispatcher implements FlowDispatcher
{
    private LinkedList  subHandles;

    /**
     * Class constructor
     */
    public SimpleFlowDispatcher() 
    {
	subHandles = new LinkedList();
    }

    /**
     * Register a flow dispatcher with this dispatcher.
     *
     * This has the effect that flow handle sub will only be presented
     * with packets dispatched to us. We require the object implementing
     * the flow dispatcher to be of type Linkable (XXX). 
     *
     * @param sub	FlowDispatcher to be added.
     */
    public void registerHandle(FlowDispatcher sub)
    {
	subHandles.append((Linkable)sub);
    }

    /**
     * Dispatch a given buffer.
     * 
     * Dispatches the buffer to all registered dispatchers
     *
     * @see FlowDispatcher#dispatchBuffer
     */
    public Buffer dispatchBuffer(Buffer buf, int offset)
    {
	Buffer exchangebuf = null;

	for (Linkable f = (Linkable)subHandles.first; 
	    f != null; f = (Linkable)f.next)
	{
	    Buffer exb = null;

	    if (f instanceof FlowHandle) {
		FlowHandle fh = (FlowHandle)f;

		if (fh.checkAlwaysTrue || fh.checkBuffer(buf, offset))
		    exb = fh.dispatchBuffer(buf, offset);
	    } else {
		FlowDispatcher fd = (FlowDispatcher)f;
		exb = fd.dispatchBuffer(buf, offset);
	    }

	    if (exb != null) {
		buf.exchanged = true;
		exchangebuf = exb;
	    }
	}

	return exchangebuf;
    }
}
