/* WARNING: This file was automatically generated from nodeos/UDPFlowManager.Java DO NOT EDIT *//* 
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries reproduce
 * the notices in supporting documentation, and (3) neither the name of the
 * University nor the Flux Group may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 * 
 * THE UNIVERSITY OF UTAH AND THE FLUX GROUP ALLOW FREE USE OF THIS SOFTWARE
 * IN ITS "AS IS" CONDITION, AND DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY
 * DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * The University requests users to return to flux-dist@cs.utah.edu any
 * improvements that they make and grant the University redistribution rights.
 */


package nodeos ;

import java.util.Vector;
import java.net.*;
import java.io.IOException;

/**
 * A helper class to dispatch UDP packets for a port to their dispatcher.
 *
 * This class is specific to the usermode implementation.
 *
 * @author Godmar Back
 *
 * @see FlowManager
 */
public class UDPFlowManager implements Runnable
{
    private DatagramSocket      udpSocket;
    private FlowDispatcher	flowDispatcher;

    /**
     * Start a new thread, receiving and dispatching packets.
     *
     * @param port	an int describing the UDP port for this manager
     * @param flowDispatcher packet destination
     */
    public UDPFlowManager(int port, FlowDispatcher flowDispatcher)
    {
	this.flowDispatcher = flowDispatcher;

        try {
            udpSocket = new DatagramSocket(port);
        } catch (SocketException se) {
            System.out.println(se + "\n");
            /* XXX throw ANRuntimeSystemException() */
        }
        new Thread(this).start();
    }

    /**
     * Receive packets and dispatch loop.
     */
    public void run()
    {
	Buffer sbuf = new Buffer(FlowManager.UDPMTU);

        for (;;) {
	    sbuf.reset();
	    sbuf.exchanged = false;
	    boolean latch = false;

            /* receive a packet */
            try { 
                DatagramPacket p = new
                    DatagramPacket(sbuf.getData(), sbuf.getLength());
                udpSocket.receive(p);
		sbuf.setLength(p.getLength());
            } catch (IOException ie) { 
                System.out.println(ie + " - ignored, continuing...");
		continue;
            }

	    Buffer b;
	    synchronized (FlowHandle.irqlock) {
		b = flowDispatcher.dispatchBuffer(sbuf, 0);
	    }
	    if (b != null)
		sbuf = b;
	    else
		System.out.println("drop packet");
        }       
    }

    /**
     * Return internal UDP socket.
     * 
     * This provides direct access to the UDP socket used.
     * XXX - this is temporary until the send question is solved.
     *
     * @return	socket used by this UDP flow manager
     */
    DatagramSocket getSocket()
    {
	return udpSocket;
    }
}

