/* 
 *  $Id: ClassCode.Java,v 1.1.1.1 1998/02/24 04:07:37 gback Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package utils;

import java.io.*;


/**
 * Obtain classfile for a class
 *
 * @author David Wetherall
 */


public class ClassCode {

  /**
   * Convert a fully qualified class name into an array
   * of its bytes that is the classfile. Search the standard
   * classpath and load the file -- can't find any other way
   */

  public static byte[] get(String cn) 
    throws Exception 
  {
    // Add a terminating ':' on to the path so that we 
    // are sure we search the last element
    String cp = System.getProperty("java.class.path") + File.pathSeparator;
    String ce = File.separator + cn.replace('.', File.separatorChar) + ".class";
    FileInputStream cfi = null;
    File cf = null;
    int last = 0;
    int pos;
    

    while ((pos = cp.indexOf(File.pathSeparatorChar, last)) > 0) {
      cf = new File(cp.substring(last, pos) + ce);
      last = pos+1;
      try {
	cfi = new FileInputStream(cf);
	break;
      } catch (FileNotFoundException e) {
	continue;
      }
    }
    
    if (cf == null || cfi == null)
      throw new Exception("code not found for "+ cn);
    
    int len = (int)cf.length();
    byte[] cb = new byte[len];

    if (cfi.read(cb) != len)
      throw new Exception("short read for "+ cn);

    return cb;
  }
  

  /** Test the class. */
  public static void main(String[] args)
    throws Exception 
  {
    byte[] code = get("utils.ClassCode");
  }
}
