/* WARNING: This file was automatically generated from utils/ClassPathReader.Java DO NOT EDIT *//*
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries reproduce
 * the notices in supporting documentation, and (3) neither the name of the
 * University nor the Flux Group may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 * 
 * THE UNIVERSITY OF UTAH AND THE FLUX GROUP ALLOW FREE USE OF THIS SOFTWARE
 * IN ITS "AS IS" CONDITION, AND DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY
 * DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * The University requests users to return to flux-dist@cs.utah.edu any
 * improvements that they make and grant the University redistribution rights.
 */
package utils ;

/**
 * This class implements the ByteCodeReader interface
 * based on a "CLASSPATH" which can be given as a parameter.
 *
 * @author Godmar Back (gback@cs.utah.edu)
 */
import java.util.*;
import java.util.zip.*;
import java.io.*;

public
class ClassPathReader implements ByteCodeReader
{
    /**
     * Components of this path
     */
    private Vector pathcomps;

    /**
     * Hashtable of already read classes
     */
    private Hashtable	classcache;

    public ClassPathReader() 
    {
	this(System.getProperty("java.class.path"));
    }

    /**
     * Build a class path from the specified path string
     */
    public ClassPathReader(String pathstr) 
    {
	classcache = new Hashtable();
	pathcomps = new Vector(5,2);

	String s = pathstr;
	int i;
	while ((i = s.indexOf(File.pathSeparatorChar)) != -1) {
	    if (i == 0)
		pathcomps.addElement(".");
	    else
		pathcomps.addElement(s.substring(0, i));
	    s = s.substring(i+1);
	}
	pathcomps.addElement(s);
    }

    /**
     * resolve a class name of the form foo.bar.baz
     *
     * @param classname fully qualified class name
     * @return byte[] array of bytes containing .class file
     */
    public byte[] readByteCode(String classname) throws Exception
    {
	// check in cache first
	byte [] p = (byte [])classcache.get(classname);

	if (p != null)
	    return p;

	BufferedInputStream file;
	String cname = classname.replace('.', '/').concat(".class");

	// iterate through all components and look for .class file
	for (int i = 0; i < pathcomps.size(); i++) {

	    String compname = (String)pathcomps.elementAt(i);
	    try {
		// a zip file ?
		ZipFile  zip   = new ZipFile(compname);
		ZipEntry entry = zip.getEntry(cname);

		// avoid throwing and catching a null pointer exception
		if (entry == null)
		    throw new Exception("not in zip file");

		file = new BufferedInputStream(zip.getInputStream(entry));

	    } catch (Exception ie0) {

		try {
		    String fn  = compname.concat("/").concat(cname);

		    // a regular file
		    file = new BufferedInputStream(new FileInputStream(fn));

		} catch (IOException ie1) {
		    continue;
		} 
	    }

	    // now try to read the file
	    p = new byte[file.available()];
	    if (file.read(p) != p.length)
		throw new Exception("Short read for " + classname);

	    classcache.put(cname, p);
	    return p;
	}

	throw new Exception("Couldn't resolve class `" 
		+ classname + "'\n" 
	    	+ " in CLASSPATH " + this);
    }

    /**
     * convert in printable form
     * 
     * @return String describing this class path
     */
    public String toString() {
	return pathcomps.toString();
    }

    /** a test */
    public static void main(String a[]) throws Exception
    {
        ClassPathReader p = new ClassPathReader();
        System.out.println(p.readByteCode("java.lang.Object"));
        System.out.println(p.readByteCode("java.io.File"));
        System.out.println(p.readByteCode("ClassPathReader"));
        System.out.println(p.readByteCode("lalala"));
    }
}
