/*
 * Copyright (c) 1998 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries reproduce
 * the notices in supporting documentation, and (3) neither the name of the
 * University nor the Flux Group may be used to endorse or promote products
 * derived from this software without specific prior written permission.
 * 
 * THE UNIVERSITY OF UTAH AND THE FLUX GROUP ALLOW FREE USE OF THIS SOFTWARE
 * IN ITS "AS IS" CONDITION, AND DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY
 * DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * The University requests users to return to flux-dist@cs.utah.edu any
 * improvements that they make and grant the University redistribution rights.
 */

/**                                                                           
 * Create a frame that when closed will exit the application. 
 * <p>
 * In ANTS 1.2, Applications were descendants of Frame.  This had             
 * the advantage that they could directly call methods such as show,          
 * pack, etc.  Secondly, the JVM would exit when the user closed a            
 * window.                                                                    
 * <p>                                                                        
 * FrameCreator is a convenience class that provides the caller
 * with a window that realizes the same behaviour.
 *
 * @author Godmar Back
 */
package utils;

import java.awt.Frame;
import java.awt.event.*;

public class FrameCreator implements WindowListener
{
  Frame window;

  public static Frame createFrame(String name) {
    Frame window = new Frame(name);
    window.addWindowListener(new FrameCreator(window));
    return window;
  }

  FrameCreator(Frame window) {
    this.window = window;
  }                                                                             

  /** Catch our window being closed as a hint bail */
  public void windowClosing(WindowEvent e) {                                    
    window.dispose();
    System.exit(0);
  }                                                                             
                                                                                
  public void windowOpened(WindowEvent e) {}                                    
  public void windowClosed(WindowEvent e) {}                                    
  public void windowIconified(WindowEvent e) {}                                 
  public void windowDeiconified(WindowEvent e) {}                               
  public void windowActivated(WindowEvent e) {}                                 
  public void windowDeactivated(WindowEvent e) {}                               
}
